@@ -14,9 +14,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 import os
+import tempfile
 
 from ironic_lib import utils as ironic_utils
+import jinja2
+from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import fileutils
@@ -29,10 +33,12 @@ from ironic.common import image_service as service
 from ironic.common import images
 from ironic.common import states
 from ironic.common import utils
+from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
 from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import image_cache
+from ironic.drivers import utils as driver_utils
 from ironic import objects
 
 LOG = logging.getLogger(__name__)
@@ -42,11 +48,11 @@ PXE_CFG_DIR_NAME = CONF.pxe.pxe_config_subdir
 DHCP_CLIENT_ID = '61'  # rfc2132
 DHCP_TFTP_SERVER_NAME = '66'  # rfc2132
 DHCP_BOOTFILE_NAME = '67'  # rfc2132
-DHCPV6_BOOTFILE_NAME = '59'  # rfc5870
+DHCPV6_BOOTFILE_NAME = '59'  # rfc5970
 # NOTE(TheJulia): adding note for the bootfile parameter
 # field as defined by RFC 5870. No practical examples seem
 # available. Neither grub2 or ipxe seem to leverage this.
-# DHCPV6_BOOTFILE_PARAMS = '60'  # rfc5870
+# DHCPV6_BOOTFILE_PARAMS = '60'  # rfc5970
 DHCP_TFTP_SERVER_ADDRESS = '150'  # rfc5859
 DHCP_IPXE_ENCAP_OPTS = '175'  # Tentatively Assigned
 DHCP_TFTP_PATH_PREFIX = '210'  # rfc5071
@@ -112,8 +118,9 @@ def _link_mac_pxe_configs(task, ipxe_enabled=False):
         create_link(_get_pxe_mac_path(port.address, client_id=client_id,
                                       ipxe_enabled=ipxe_enabled))
         # Grub2 MAC address only
-        create_link(_get_pxe_grub_mac_path(port.address,
-                                           ipxe_enabled=ipxe_enabled))
+        for path in _get_pxe_grub_mac_path(port.address,
+                                           ipxe_enabled=ipxe_enabled):
+            create_link(path)
 
 
 def _link_ip_address_pxe_configs(task, ipxe_enabled=False):
@@ -156,7 +163,9 @@ def _link_ip_address_pxe_configs(task, ipxe_enabled=False):
 
 def _get_pxe_grub_mac_path(mac, ipxe_enabled=False):
     root_dir = get_ipxe_root_dir() if ipxe_enabled else get_root_dir()
-    return os.path.join(root_dir, mac + '.conf')
+    yield os.path.join(root_dir, "%s-%s-%s" %
+                       ("grub.cfg", "01", mac.replace(':', "-").lower()))
+    yield os.path.join(root_dir, mac + '.conf')
 
 
 def _get_pxe_mac_path(mac, delimiter='-', client_id=None,
@@ -244,6 +253,54 @@ def get_pxe_config_file_path(node_uuid, ipxe_enabled=False):
         return os.path.join(get_root_dir(), node_uuid, 'config')
 
 
+def get_file_path_from_label(node_uuid, root_dir, label):
+    """Generate absolute paths to various images from their name(label)
+
+    This method generates absolute file system path on the conductor where
+    various images need to be placed. For example the kickstart template, file
+    and stage2 squashfs.img needs to be placed in the ipxe_root_dir since they
+    will be transferred by anaconda ramdisk over http(s). The generated paths
+    will be added to the image_info dictionary as values.
+
+    :param node_uuid: the UUID of the node
+    :param root_dir: Directory in which the image must be placed
+    :param label: Name of the image
+    """
+    if label == 'ks_template':
+        return os.path.join(get_ipxe_root_dir(), node_uuid, 'ks.cfg.template')
+    elif label == 'ks_cfg':
+        return os.path.join(get_ipxe_root_dir(), node_uuid, 'ks.cfg')
+    elif label == 'stage2':
+        return os.path.join(get_ipxe_root_dir(), node_uuid, 'LiveOS',
+                            'squashfs.img')
+    else:
+        return os.path.join(root_dir, node_uuid, label)
+
+
+def get_http_url_path_from_label(http_url, node_uuid, label):
+    """Generate http url path to various image artifacts
+
+    This method generates http(s) urls for various image artifacts int the
+    webserver root. The generated urls will be added to the pxe_options dict
+    and used to render pxe/ipxe configuration templates.
+
+    :param http_url: URL to access the root of the webserver
+    :param node_uuid: the UUID of the node
+    :param label: Name of the image
+    """
+    if label == 'ks_template':
+        return '/'.join([http_url, node_uuid, 'ks.cfg.template'])
+    elif label == 'ks_cfg':
+        return '/'.join([http_url, node_uuid, 'ks.cfg'])
+    elif label == 'stage2':
+        # we store stage2 in http_root/node_uuid/LiveOS/squashfs.img
+        # Specifying http://host/node_uuid as stage2 url will make anaconda
+        # automatically load the squashfs.img from LiveOS directory.
+        return '/'.join([http_url, node_uuid])
+    else:
+        return '/'.join([http_url, node_uuid, label])
+
+
 def create_pxe_config(task, pxe_options, template=None, ipxe_enabled=False):
     """Generate PXE configuration file and MAC address links for it.
 
@@ -345,8 +402,6 @@ def clean_up_pxe_config(task, ipxe_enabled=False):
     if is_uefi_boot_mode and not ipxe_enabled:
         api = dhcp_factory.DHCPFactory().provider
         ip_addresses = api.get_ip_addresses(task)
-        if not ip_addresses:
-            return
 
         for port_ip_address in ip_addresses:
             try:
@@ -366,8 +421,9 @@ def clean_up_pxe_config(task, ipxe_enabled=False):
             _get_pxe_mac_path(port.address, client_id=client_id,
                               ipxe_enabled=ipxe_enabled))
         # Grub2 MAC address based confiuration
-        ironic_utils.unlink_without_raise(
-            _get_pxe_grub_mac_path(port.address, ipxe_enabled=ipxe_enabled))
+        for path in _get_pxe_grub_mac_path(port.address,
+                                           ipxe_enabled=ipxe_enabled):
+            ironic_utils.unlink_without_raise(path)
     if ipxe_enabled:
         utils.rmtree_without_raise(os.path.join(get_ipxe_root_dir(),
                                                 task.node.uuid))
@@ -628,6 +684,13 @@ def get_instance_image_info(task, ipxe_enabled=False):
     else:
         root_dir = get_root_dir()
     i_info = node.instance_info
+    if i_info.get('boot_iso'):
+        image_info['boot_iso'] = (
+            i_info['boot_iso'],
+            os.path.join(root_dir, node.uuid, 'boot_iso'))
+
+        return image_info
+
     labels = ('kernel', 'ramdisk')
     d_info = deploy_utils.get_image_instance_info(node)
     if not (i_info.get('kernel') and i_info.get('ramdisk')):
@@ -638,10 +701,38 @@ def get_instance_image_info(task, ipxe_enabled=False):
         node.instance_info = i_info
         node.save()
 
-    for label in labels:
+    anaconda_labels = ()
+    if deploy_utils.get_boot_option(node) == 'kickstart':
+        # stage2  - Installer stage2 squashfs image
+        # ks_template - Anaconda kickstart template
+        # ks_cfg - rendered ks_template
+        anaconda_labels = ('stage2', 'ks_template', 'ks_cfg')
+        if not (i_info.get('stage2') and i_info.get('ks_template')):
+            iproperties = glance_service.show(
+                d_info['image_source']
+            )['properties']
+            for label in anaconda_labels:
+                # ks_template is an optional property on the image
+                if (label == 'ks_template'
+                        and not iproperties.get('ks_template')):
+                    i_info[label] = CONF.anaconda.default_ks_template
+                elif label == 'ks_cfg':
+                    i_info[label] = ''
+                elif label == 'stage2' and 'stage2_id' not in iproperties:
+                    msg = ("stage2_id property missing on the image. "
+                           "The anaconda deploy interface requires stage2_id "
+                           "property to be associated with the os image. ")
+                    raise exception.ImageUnacceptable(msg)
+                else:
+                    i_info[label] = str(iproperties['stage2_id'])
+
+            node.instance_info = i_info
+            node.save()
+
+    for label in labels + anaconda_labels:
         image_info[label] = (
             i_info[label],
-            os.path.join(root_dir, node.uuid, label)
+            get_file_path_from_label(node.uuid, root_dir, label)
         )
 
     return image_info
@@ -701,15 +792,18 @@ def build_instance_pxe_options(task, pxe_info, ipxe_enabled=False):
     node = task.node
 
     for label, option in (('kernel', 'aki_path'),
-                          ('ramdisk', 'ari_path')):
+                          ('ramdisk', 'ari_path'),
+                          ('stage2', 'stage2_url'),
+                          ('ks_template', 'ks_template_path'),
+                          ('ks_cfg', 'ks_cfg_url')):
         if label in pxe_info:
-            if ipxe_enabled:
+            if ipxe_enabled or label in ('stage2', 'ks_template', 'ks_cfg'):
                 # NOTE(pas-ha) do not use Swift TempURLs for kernel and
                 # ramdisk of user image when boot_option is not local,
                 # as this breaks instance reboot later when temp urls
                 # have timed out.
-                pxe_opts[option] = '/'.join(
-                    [CONF.deploy.http_url, node.uuid, label])
+                pxe_opts[option] = get_http_url_path_from_label(
+                    CONF.deploy.http_url, node.uuid, label)
             else:
                 # It is possible that we don't have kernel/ramdisk or even
                 # image_source to determine if it's a whole disk image or not.
@@ -726,19 +820,32 @@ def build_instance_pxe_options(task, pxe_info, ipxe_enabled=False):
         pxe_opts['ramdisk_opts'] = i_info['ramdisk_kernel_arguments']
     except KeyError:
         pass
+    try:
+        # TODO(TheJulia): Boot iso should change at a later point
+        # if we serve more than just as a pass-through.
+        if i_info.get('boot_iso'):
+            pxe_opts['boot_iso_url'] = '/'.join(
+                [CONF.deploy.http_url, node.uuid, 'boot_iso'])
+    except KeyError:
+        pass
 
     return pxe_opts
 
 
-def build_extra_pxe_options(ramdisk_params=None):
+def build_extra_pxe_options(task, ramdisk_params=None):
+    pxe_append_params = driver_utils.get_kernel_append_params(
+        task.node, default=CONF.pxe.kernel_append_params)
     # Enable debug in IPA according to CONF.debug if it was not
     # specified yet
-    pxe_append_params = CONF.pxe.pxe_append_params
     if CONF.debug and 'ipa-debug' not in pxe_append_params:
         pxe_append_params += ' ipa-debug=1'
     if ramdisk_params:
-        pxe_append_params += ' ' + ' '.join('%s=%s' % tpl
-                                            for tpl in ramdisk_params.items())
+        pxe_append_params += ' ' + ' '.join(
+            ('%s=%s' % tpl) if tpl[1] is not None else tpl[0]
+            for tpl in ramdisk_params.items())
+    if task and task.context.global_id:
+        pxe_append_params += (
+            ' ipa-global-request-id=%s' % task.context.global_id)
 
     return {'pxe_append_params': pxe_append_params,
             'tftp_server': CONF.pxe.tftp_server,
@@ -786,7 +893,7 @@ def build_pxe_config_options(task, pxe_info, service=False,
     pxe_options.update(build_instance_pxe_options(task, pxe_info,
                                                   ipxe_enabled=ipxe_enabled))
 
-    pxe_options.update(build_extra_pxe_options(ramdisk_params))
+    pxe_options.update(build_extra_pxe_options(task, ramdisk_params))
 
     return pxe_options
 
@@ -795,7 +902,8 @@ def build_service_pxe_config(task, instance_image_info,
                              root_uuid_or_disk_id,
                              ramdisk_boot=False,
                              ipxe_enabled=False,
-                             is_whole_disk_image=None):
+                             is_whole_disk_image=None,
+                             anaconda_boot=False):
     node = task.node
     pxe_config_path = get_pxe_config_file_path(node.uuid,
                                                ipxe_enabled=ipxe_enabled)
@@ -829,7 +937,38 @@ def build_service_pxe_config(task, instance_image_info,
         is_whole_disk_image,
         deploy_utils.is_trusted_boot_requested(node),
         deploy_utils.is_iscsi_boot(task), ramdisk_boot,
-        ipxe_enabled=ipxe_enabled)
+        ipxe_enabled=ipxe_enabled, anaconda_boot=anaconda_boot)
+
+
+def _build_heartbeat_url(node_uuid):
+
+    api_version = 'v1'
+    heartbeat_api = '%s/heartbeat/{node_uuid}' % api_version
+    path = heartbeat_api.format(node_uuid=node_uuid)
+    return "/".join([deploy_utils.get_ironic_api_url(), path])
+
+
+def build_kickstart_config_options(task):
+    """Build the kickstart template options for a node
+
+    This method builds the kickstart template options for a node,
+    given all the required parameters.
+
+    The options should then be passed to pxe_utils.create_kickstart_config to
+    create the actual config files.
+
+    :param task: A TaskManager object
+    :returns: A dictionary of kickstart options to be used in the kickstart
+              template.
+    """
+    ks_options = {}
+    node = task.node
+    manager_utils.add_secret_token(node, pregenerated=True)
+    node.save()
+    ks_options['liveimg_url'] = node.instance_info['image_url']
+    ks_options['agent_token'] = node.driver_internal_info['agent_secret_token']
+    ks_options['heartbeat_url'] = _build_heartbeat_url(node.uuid)
+    return ks_options
 
 
 def get_volume_pxe_options(task):
@@ -846,12 +985,7 @@ def get_volume_pxe_options(task):
             return prop
         return __return_item_or_first_if_list(properties.get(key + 's', ''))
 
-    def __generate_iscsi_url(properties):
-        """Returns iscsi url."""
-        portal = __get_property(properties, 'target_portal')
-        iqn = __get_property(properties, 'target_iqn')
-        lun = __get_property(properties, 'target_lun')
-
+    def __format_portal(portal, iqn, lun):
         if ':' in portal:
             host, port = portal.split(':')
         else:
@@ -860,6 +994,20 @@ def get_volume_pxe_options(task):
         return ("iscsi:%(host)s::%(port)s:%(lun)s:%(iqn)s" %
                 {'host': host, 'port': port, 'lun': lun, 'iqn': iqn})
 
+    def __generate_iscsi_url(properties):
+        """Returns iscsi url."""
+        iqn = __get_property(properties, 'target_iqn')
+        lun = __get_property(properties, 'target_lun')
+        if 'target_portals' in properties:
+            portals = properties.get('target_portals')
+            formatted_portals = []
+            for portal in portals:
+                formatted_portals.append(__format_portal(portal, iqn, lun))
+            return ' '.join(formatted_portals)
+        else:
+            portal = __get_property(properties, 'target_portal')
+            return __format_portal(portal, iqn, lun)
+
     pxe_options = {}
     node = task.node
     boot_volume = node.driver_internal_info.get('boot_from_volume')
@@ -922,10 +1070,71 @@ def validate_boot_parameters_for_trusted_boot(node):
         raise exception.InvalidParameterValue(msg)
 
 
+def validate_kickstart_template(ks_template):
+    """Validate the kickstart template
+
+    :param ks_template: Path to the kickstart template
+    :raises: InvalidKickstartTemplate
+    """
+    ks_options = {'liveimg_url': 'fake_image_url',
+                  'agent_token': 'fake_token',
+                  'heartbeat_url': 'fake_heartbeat_url'}
+    params = {'ks_options': ks_options}
+    try:
+        rendered_tmpl = utils.render_template(ks_template, params, strict=True)
+    except jinja2.exceptions.UndefinedError as exc:
+        msg = (_("The kickstart template includes a variable that is not "
+                 "a valid kickstart option. Rendering the template returned "
+                 " %(msg)s. The valid options are %(valid_options)s.") %
+               {'msg': exc.message,
+                'valid_options': ','.join(ks_options.keys())})
+        raise exception.InvalidKickstartTemplate(msg)
+
+    missing_required_options = []
+    for var, value in ks_options.items():
+        if rendered_tmpl.find(value) == -1:
+            missing_required_options.append(var)
+    if missing_required_options:
+        msg = (_("Following required kickstart option variables are missing "
+                 "from the kickstart template: %(missing_opts)s.") %
+               {'missing_opts': ','.join(missing_required_options)})
+        raise exception.InvalidKickstartTemplate(msg)
+    return rendered_tmpl
+
+
+def validate_kickstart_file(ks_cfg):
+    """Check if the kickstart file is valid
+
+    :param ks_cfg: Contents of kickstart file to validate
+    :raises: InvalidKickstartFile
+    """
+    if not os.path.isfile('/usr/bin/ksvalidator'):
+        LOG.warning(
+            "Unable to validate the kickstart file as ksvalidator binary is "
+            "missing. Please install pykickstart package to enable "
+            "validation of kickstart file."
+        )
+        return
+
+    with tempfile.NamedTemporaryFile(
+            dir=CONF.tempdir, suffix='.cfg') as ks_file:
+        ks_file.writelines(ks_cfg)
+        try:
+            result = utils.execute(
+                'ksvalidator', ks_file.name, check_on_exit=[0], attempts=1
+            )
+        except processutils.ProcessExecutionError:
+            msg = _(("The kickstart file generated does not pass validation. "
+                     "The ksvalidator tool returned following error(s): %s") %
+                    (result))
+            raise exception.InvalidKickstartFile(msg)
+
+
 def prepare_instance_pxe_config(task, image_info,
                                 iscsi_boot=False,
                                 ramdisk_boot=False,
-                                ipxe_enabled=False):
+                                ipxe_enabled=False,
+                                anaconda_boot=False):
     """Prepares the config file for PXE boot
 
     :param task: a task from TaskManager.
@@ -935,9 +1144,9 @@ def prepare_instance_pxe_config(task, image_info,
     :param ramdisk_boot: if the boot is to a ramdisk configuration.
     :param ipxe_enabled: Default false boolean to indicate if ipxe
                          is in use by the caller.
+    :param anaconda_boot: if the boot is to a anaconda ramdisk configuration.
     :returns: None
     """
-
     node = task.node
     # Generate options for both IPv4 and IPv6, and they can be
     # filtered down later based upon the port options.
@@ -955,7 +1164,7 @@ def prepare_instance_pxe_config(task, image_info,
         node.uuid, ipxe_enabled=ipxe_enabled)
     if not os.path.isfile(pxe_config_path):
         pxe_options = build_pxe_config_options(
-            task, image_info, service=ramdisk_boot,
+            task, image_info, service=ramdisk_boot or anaconda_boot,
             ipxe_enabled=ipxe_enabled)
         if ipxe_enabled:
             pxe_config_template = (
@@ -970,7 +1179,23 @@ def prepare_instance_pxe_config(task, image_info,
         pxe_config_path, None,
         boot_mode_utils.get_boot_mode(node), False,
         iscsi_boot=iscsi_boot, ramdisk_boot=ramdisk_boot,
-        ipxe_enabled=ipxe_enabled)
+        ipxe_enabled=ipxe_enabled, anaconda_boot=anaconda_boot)
+
+
+def prepare_instance_kickstart_config(task, image_info, anaconda_boot=False):
+    """Prepare to boot anaconda ramdisk by generating kickstart file
+
+    :param task: a task from TaskManager.
+    :param image_info: a dict of values of instance image
+                       metadata to set on the configuration file.
+    :param anaconda_boot: if the boot is to a anaconda ramdisk configuration.
+    """
+    if not anaconda_boot:
+        return
+    ks_options = build_kickstart_config_options(task)
+    kickstart_template = image_info['ks_template'][1]
+    ks_cfg = utils.render_template(kickstart_template, ks_options)
+    utils.write_to_file(image_info['ks_cfg'][1], ks_cfg)
 
 
 @image_cache.cleanup(priority=25)
@@ -989,14 +1214,30 @@ def cache_ramdisk_kernel(task, pxe_info, ipxe_enabled=False):
     """Fetch the necessary kernels and ramdisks for the instance."""
     ctx = task.context
     node = task.node
+    t_pxe_info = copy.copy(pxe_info)
     if ipxe_enabled:
         path = os.path.join(get_ipxe_root_dir(), node.uuid)
     else:
         path = os.path.join(get_root_dir(), node.uuid)
     fileutils.ensure_tree(path)
+    # anconda deploy will have 'stage2' as one of the labels in pxe_info dict
+    if 'stage2' in pxe_info.keys():
+        # stage2  will be stored in ipxe http directory. So make sure they
+        # exist.
+        fileutils.ensure_tree(
+            get_file_path_from_label(
+                node.uuid,
+                get_ipxe_root_dir(),
+                'stage2'
+            )
+        )
+        # ks_cfg is rendered later by the driver using ks_template. It cannot
+        # be fetched and cached.
+        t_pxe_info.pop('ks_cfg')
+
     LOG.debug("Fetching necessary kernel and ramdisk for node %s",
               node.uuid)
-    deploy_utils.fetch_images(ctx, TFTPImageCache(), list(pxe_info.values()),
+    deploy_utils.fetch_images(ctx, TFTPImageCache(), list(t_pxe_info.values()),
                               CONF.force_raw_images)
 
 
