@@ -344,10 +344,11 @@ class Connection(object, metaclass=abc.ABCMeta):
         """
 
     @abc.abstractmethod
-    def get_portgroup_by_address(self, address):
+    def get_portgroup_by_address(self, address, project=None):
         """Return a network portgroup representation.
 
         :param address: The MAC address of a portgroup.
+        :param project: A node owner or lessee to filter by.
         :returns: A portgroup.
         :raises: PortgroupNotFound
         """
@@ -363,7 +364,8 @@ class Connection(object, metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def get_portgroup_list(self, limit=None, marker=None,
-                           sort_key=None, sort_dir=None):
+                           sort_key=None, sort_dir=None,
+                           project=None):
         """Return a list of portgroups.
 
         :param limit: Maximum number of portgroups to return.
@@ -372,12 +374,14 @@ class Connection(object, metaclass=abc.ABCMeta):
         :param sort_key: Attribute by which results should be sorted.
         :param sort_dir: Direction in which results should be sorted.
                          (asc, desc)
+        :param project: A node owner or lessee to filter by.
         :returns: A list of portgroups.
         """
 
     @abc.abstractmethod
     def get_portgroups_by_node_id(self, node_id, limit=None, marker=None,
-                                  sort_key=None, sort_dir=None):
+                                  sort_key=None, sort_dir=None,
+                                  project=None):
         """List all the portgroups for a given node.
 
         :param node_id: The integer node ID.
@@ -387,6 +391,7 @@ class Connection(object, metaclass=abc.ABCMeta):
         :param sort_key: Attribute by which results should be sorted
         :param sort_dir: Direction in which results should be sorted
                          (asc, desc)
+        :param project: A node owner or lessee to filter by.
         :returns: A list of portgroups.
         """
 
@@ -709,7 +714,8 @@ class Connection(object, metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def get_volume_connector_list(self, limit=None, marker=None,
-                                  sort_key=None, sort_dir=None):
+                                  sort_key=None, sort_dir=None,
+                                  project=None):
         """Return a list of volume connectors.
 
         :param limit: Maximum number of volume connectors to return.
@@ -718,6 +724,8 @@ class Connection(object, metaclass=abc.ABCMeta):
         :param sort_key: Attribute by which results should be sorted.
         :param sort_dir: Direction in which results should be sorted.
                          (asc, desc)
+        :param project: The associated node project to search with.
+        :returns: a list of :class:`VolumeConnector` objects
         :returns: A list of volume connectors.
         :raises: InvalidParameterValue If sort_key does not exist.
         """
@@ -745,7 +753,7 @@ class Connection(object, metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def get_volume_connectors_by_node_id(self, node_id, limit=None,
                                          marker=None, sort_key=None,
-                                         sort_dir=None):
+                                         sort_dir=None, project=None):
         """List all the volume connectors for a given node.
 
         :param node_id: The integer node ID.
@@ -755,6 +763,8 @@ class Connection(object, metaclass=abc.ABCMeta):
         :param sort_key: Attribute by which results should be sorted
         :param sort_dir: Direction in which results should be sorted
                          (asc, desc)
+        :param project: The associated node project to search with.
+        :returns: a list of :class:`VolumeConnector` objects
         :returns: A list of volume connectors.
         :raises: InvalidParameterValue If sort_key does not exist.
         """
@@ -808,7 +818,8 @@ class Connection(object, metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def get_volume_target_list(self, limit=None, marker=None,
-                               sort_key=None, sort_dir=None):
+                               sort_key=None, sort_dir=None,
+                               project=None):
         """Return a list of volume targets.
 
         :param limit: Maximum number of volume targets to return.
@@ -817,6 +828,8 @@ class Connection(object, metaclass=abc.ABCMeta):
         :param sort_key: Attribute by which results should be sorted.
         :param sort_dir: direction in which results should be sorted.
                          (asc, desc)
+        :param project: The associated node project to search with.
+        :returns: a list of :class:`VolumeConnector` objects
         :returns: A list of volume targets.
         :raises: InvalidParameterValue if sort_key does not exist.
         """
@@ -844,7 +857,7 @@ class Connection(object, metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def get_volume_targets_by_node_id(self, node_id, limit=None,
                                       marker=None, sort_key=None,
-                                      sort_dir=None):
+                                      sort_dir=None, project=None):
         """List all the volume targets for a given node.
 
         :param node_id: The integer node ID.
@@ -854,6 +867,8 @@ class Connection(object, metaclass=abc.ABCMeta):
         :param sort_key: Attribute by which results should be sorted
         :param sort_dir: direction in which results should be sorted
                          (asc, desc)
+        :param project: The associated node project to search with.
+        :returns: a list of :class:`VolumeConnector` objects
         :returns: A list of volume targets.
         :raises: InvalidParameterValue if sort_key does not exist.
         """
@@ -861,7 +876,7 @@ class Connection(object, metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def get_volume_targets_by_volume_id(self, volume_id, limit=None,
                                         marker=None, sort_key=None,
-                                        sort_dir=None):
+                                        sort_dir=None, project=None):
         """List all the volume targets for a given volume id.
 
         :param volume_id: The UUID of the volume.
@@ -958,18 +973,6 @@ class Connection(object, metaclass=abc.ABCMeta):
                   of migrated objects.
         """
 
-    @abc.abstractmethod
-    def migrate_from_iscsi_deploy(self, context, max_count):
-        """Tries to migrate away from the iscsi deploy interface.
-
-        :param context: the admin context
-        :param max_count: The maximum number of objects to migrate. Must be
-                          >= 0. If zero, all the objects will be migrated.
-        :returns: A 2-tuple, 1. the total number of objects that need to be
-                  migrated (at the beginning of this call) and 2. the number
-                  of migrated objects.
-        """
-
     @abc.abstractmethod
     def set_node_traits(self, node_id, traits, version):
         """Replace all of the node traits with specified list of traits.
@@ -1051,10 +1054,12 @@ class Connection(object, metaclass=abc.ABCMeta):
                           {
                            'name': String,
                            'value': String,
+                           additional settings from BIOS registry
                           },
                           {
                            'name': String,
                            'value': String,
+                           additional settings from BIOS registry
                           },
                           ...
                         ]
@@ -1078,10 +1083,12 @@ class Connection(object, metaclass=abc.ABCMeta):
                           {
                            'name': String,
                            'value': String,
+                           additional settings from BIOS registry
                           },
                           {
                            'name': String,
                            'value': String,
+                           additional settings from BIOS registry
                           },
                           ...
                         ]
