@@ -162,6 +162,27 @@ def add_port_filter_by_node_project(query, value):
                         | (models.Node.lessee == value))
 
 
+def add_portgroup_filter_by_node_project(query, value):
+    query = query.join(models.Node,
+                       models.Portgroup.node_id == models.Node.id)
+    return query.filter((models.Node.owner == value)
+                        | (models.Node.lessee == value))
+
+
+def add_volume_conn_filter_by_node_project(query, value):
+    query = query.join(models.Node,
+                       models.VolumeConnector.node_id == models.Node.id)
+    return query.filter((models.Node.owner == value)
+                        | (models.Node.lessee == value))
+
+
+def add_volume_target_filter_by_node_project(query, value):
+    query = query.join(models.Node,
+                       models.VolumeTarget.node_id == models.Node.id)
+    return query.filter((models.Node.owner == value)
+                        | (models.Node.lessee == value))
+
+
 def add_portgroup_filter(query, value):
     """Adds a portgroup-specific filter to a query.
 
@@ -295,7 +316,7 @@ class Connection(api.Connection):
     _NODE_QUERY_FIELDS = {'console_enabled', 'maintenance', 'retired',
                           'driver', 'resource_class', 'provision_state',
                           'uuid', 'id', 'fault', 'conductor_group',
-                          'owner', 'lessee'}
+                          'owner', 'lessee', 'instance_uuid'}
     _NODE_IN_QUERY_FIELDS = {'%s_in' % field: field
                              for field in ('uuid', 'provision_state')}
     _NODE_NON_NULL_FILTERS = {'associated': 'instance_uuid',
@@ -418,7 +439,7 @@ class Connection(api.Connection):
         return _paginate_query(models.Node, limit, marker,
                                sort_key, sort_dir, query)
 
-    def check_node_list(self, idents):
+    def check_node_list(self, idents, project=None):
         mapping = {}
         if idents:
             idents = set(idents)
@@ -438,6 +459,10 @@ class Connection(api.Connection):
             sql.or_(models.Node.uuid.in_(uuids),
                     models.Node.name.in_(names))
         )
+        if project:
+            query = query.filter((models.Node.owner == project)
+                                 | (models.Node.lessee == project))
+
         for row in query:
             if row[0] in idents:
                 mapping[row[0]] = row[0]
@@ -796,8 +821,10 @@ class Connection(api.Connection):
             if count == 0:
                 raise exception.PortNotFound(port=port_id)
 
-    def get_portgroup_by_id(self, portgroup_id):
+    def get_portgroup_by_id(self, portgroup_id, project=None):
         query = model_query(models.Portgroup).filter_by(id=portgroup_id)
+        if project:
+            query = add_portgroup_filter_by_node_project(query, project)
         try:
             return query.one()
         except NoResultFound:
@@ -810,8 +837,10 @@ class Connection(api.Connection):
         except NoResultFound:
             raise exception.PortgroupNotFound(portgroup=portgroup_uuid)
 
-    def get_portgroup_by_address(self, address):
+    def get_portgroup_by_address(self, address, project=None):
         query = model_query(models.Portgroup).filter_by(address=address)
+        if project:
+            query = add_portgroup_filter_by_node_project(query, project)
         try:
             return query.one()
         except NoResultFound:
@@ -825,14 +854,19 @@ class Connection(api.Connection):
             raise exception.PortgroupNotFound(portgroup=name)
 
     def get_portgroup_list(self, limit=None, marker=None,
-                           sort_key=None, sort_dir=None):
+                           sort_key=None, sort_dir=None, project=None):
+        query = model_query(models.Portgroup)
+        if project:
+            query = add_portgroup_filter_by_node_project(query, project)
         return _paginate_query(models.Portgroup, limit, marker,
-                               sort_key, sort_dir)
+                               sort_key, sort_dir, query)
 
     def get_portgroups_by_node_id(self, node_id, limit=None, marker=None,
-                                  sort_key=None, sort_dir=None):
+                                  sort_key=None, sort_dir=None, project=None):
         query = model_query(models.Portgroup)
         query = query.filter_by(node_id=node_id)
+        if project:
+            query = add_portgroup_filter_by_node_project(query, project)
         return _paginate_query(models.Portgroup, limit, marker,
                                sort_key, sort_dir, query)
 
@@ -1219,9 +1253,12 @@ class Connection(api.Connection):
                 % addresses)
 
     def get_volume_connector_list(self, limit=None, marker=None,
-                                  sort_key=None, sort_dir=None):
+                                  sort_key=None, sort_dir=None, project=None):
+        query = model_query(models.VolumeConnector)
+        if project:
+            query = add_volume_conn_filter_by_node_project(query, project)
         return _paginate_query(models.VolumeConnector, limit, marker,
-                               sort_key, sort_dir)
+                               sort_key, sort_dir, query)
 
     def get_volume_connector_by_id(self, db_id):
         query = model_query(models.VolumeConnector).filter_by(id=db_id)
@@ -1240,8 +1277,10 @@ class Connection(api.Connection):
 
     def get_volume_connectors_by_node_id(self, node_id, limit=None,
                                          marker=None, sort_key=None,
-                                         sort_dir=None):
+                                         sort_dir=None, project=None):
         query = model_query(models.VolumeConnector).filter_by(node_id=node_id)
+        if project:
+            add_volume_conn_filter_by_node_project(query, project)
         return _paginate_query(models.VolumeConnector, limit, marker,
                                sort_key, sort_dir, query)
 
@@ -1299,9 +1338,12 @@ class Connection(api.Connection):
                 raise exception.VolumeConnectorNotFound(connector=ident)
 
     def get_volume_target_list(self, limit=None, marker=None,
-                               sort_key=None, sort_dir=None):
+                               sort_key=None, sort_dir=None, project=None):
+        query = model_query(models.VolumeTarget)
+        if project:
+            query = add_volume_target_filter_by_node_project(query, project)
         return _paginate_query(models.VolumeTarget, limit, marker,
-                               sort_key, sort_dir)
+                               sort_key, sort_dir, query)
 
     def get_volume_target_by_id(self, db_id):
         query = model_query(models.VolumeTarget).filter_by(id=db_id)
@@ -1318,15 +1360,20 @@ class Connection(api.Connection):
             raise exception.VolumeTargetNotFound(target=uuid)
 
     def get_volume_targets_by_node_id(self, node_id, limit=None, marker=None,
-                                      sort_key=None, sort_dir=None):
+                                      sort_key=None, sort_dir=None,
+                                      project=None):
         query = model_query(models.VolumeTarget).filter_by(node_id=node_id)
+        if project:
+            add_volume_target_filter_by_node_project(query, project)
         return _paginate_query(models.VolumeTarget, limit, marker, sort_key,
                                sort_dir, query)
 
     def get_volume_targets_by_volume_id(self, volume_id, limit=None,
                                         marker=None, sort_key=None,
-                                        sort_dir=None):
+                                        sort_dir=None, project=None):
         query = model_query(models.VolumeTarget).filter_by(volume_id=volume_id)
+        if project:
+            query = add_volume_target_filter_by_node_project(query, project)
         return _paginate_query(models.VolumeTarget, limit, marker, sort_key,
                                sort_dir, query)
 
@@ -1531,59 +1578,6 @@ class Connection(api.Connection):
 
         return total_to_migrate, total_migrated
 
-    @oslo_db_api.retry_on_deadlock
-    def migrate_from_iscsi_deploy(self, context, max_count, force=False):
-        """Tries to migrate away from the iscsi deploy interface.
-
-        :param context: the admin context
-        :param max_count: The maximum number of objects to migrate. Must be
-                          >= 0. If zero, all the objects will be migrated.
-        :returns: A 2-tuple, 1. the total number of objects that need to be
-                  migrated (at the beginning of this call) and 2. the number
-                  of migrated objects.
-        """
-        # TODO(dtantsur): maybe change to force=True by default in W?
-        if not force:
-            if 'direct' not in CONF.enabled_deploy_interfaces:
-                LOG.warning('The direct deploy interface is not enabled, will '
-                            'not migrate nodes to it. Run with --option '
-                            'force=true to override.')
-                return 0, 0
-
-            if CONF.default_deploy_interface == 'iscsi':
-                LOG.warning('The iscsi deploy interface is the default, will '
-                            'not migrate nodes away from it. Run with '
-                            '--option force=true to override.')
-                return 0, 0
-
-            if CONF.agent.image_download_source == 'swift':
-                LOG.warning('The direct deploy interface is using swift, will '
-                            'not migrate nodes to it. Run with --option '
-                            'force=true to override.')
-                return 0, 0
-
-        total_to_migrate = (model_query(models.Node)
-                            .filter_by(deploy_interface='iscsi')
-                            .count())
-        if not total_to_migrate:
-            return 0, 0
-
-        max_to_migrate = max_count or total_to_migrate
-
-        with _session_for_write():
-            query = (model_query(models.Node.id)
-                     .filter_by(deploy_interface='iscsi')
-                     .slice(0, max_to_migrate))
-            ids = [row[0] for row in query]
-
-            num_migrated = (model_query(models.Node)
-                            .filter_by(deploy_interface='iscsi')
-                            .filter(models.Node.id.in_(ids))
-                            .update({'deploy_interface': 'direct'},
-                                    synchronize_session=False))
-
-        return total_to_migrate, num_migrated
-
     @staticmethod
     def _verify_max_traits_per_node(node_id, num_traits):
         """Verify that an operation would not exceed the per-node trait limit.
@@ -1681,6 +1675,15 @@ class Connection(api.Connection):
                         node_id=node_id,
                         name=setting['name'],
                         value=setting['value'],
+                        attribute_type=setting.get('attribute_type'),
+                        allowable_values=setting.get('allowable_values'),
+                        lower_bound=setting.get('lower_bound'),
+                        max_length=setting.get('max_length'),
+                        min_length=setting.get('min_length'),
+                        read_only=setting.get('read_only'),
+                        reset_required=setting.get('reset_required'),
+                        unique=setting.get('unique'),
+                        upper_bound=setting.get('upper_bound'),
                         version=version)
                     bios_settings.append(bios_setting)
                     session.add(bios_setting)
@@ -1701,6 +1704,18 @@ class Connection(api.Connection):
                         node_id=node_id, name=setting['name'])
                     ref = query.one()
                     ref.update({'value': setting['value'],
+                                'attribute_type':
+                                setting.get('attribute_type'),
+                                'allowable_values':
+                                setting.get('allowable_values'),
+                                'lower_bound': setting.get('lower_bound'),
+                                'max_length': setting.get('max_length'),
+                                'min_length': setting.get('min_length'),
+                                'read_only': setting.get('read_only'),
+                                'reset_required':
+                                setting.get('reset_required'),
+                                'unique': setting.get('unique'),
+                                'upper_bound': setting.get('upper_bound'),
                                 'version': version})
                     bios_settings.append(ref)
                 session.flush()
