@@ -22,7 +22,7 @@ from ironic_lib import metrics_utils
 from oslo_log import log
 from oslo_utils import strutils
 from oslo_utils import timeutils
-import retrying
+import tenacity
 
 from ironic.common import boot_devices
 from ironic.common import dhcp_factory
@@ -203,12 +203,14 @@ def _post_step_reboot(task, step_type):
                 'node': task.node.uuid,
                 'err': e,
                 'type': step_type})
-        LOG.error(msg, exc_info=not isinstance(e, exception.IronicException))
+        traceback = not isinstance(e, exception.IronicException)
         # do not set cleaning_reboot if we didn't reboot
         if step_type == 'clean':
-            manager_utils.cleaning_error_handler(task, msg)
+            manager_utils.cleaning_error_handler(task, msg,
+                                                 traceback=traceback)
         else:
-            manager_utils.deploying_error_handler(task, msg)
+            manager_utils.deploying_error_handler(task, msg,
+                                                  traceback=traceback)
         return
 
     # Signify that we've rebooted
@@ -387,67 +389,20 @@ def execute_clean_step(task, step):
     return execute_step(task, step, 'clean')
 
 
-def _step_failure_handler(task, msg, step_type):
+def _step_failure_handler(task, msg, step_type, traceback=False):
     driver_utils.collect_ramdisk_logs(
         task.node, label='cleaning' if step_type == 'clean' else None)
     if step_type == 'clean':
-        manager_utils.cleaning_error_handler(task, msg)
+        manager_utils.cleaning_error_handler(task, msg, traceback=traceback)
     else:
-        manager_utils.deploying_error_handler(task, msg)
+        manager_utils.deploying_error_handler(task, msg, traceback=traceback)
 
 
 class HeartbeatMixin(object):
     """Mixin class implementing heartbeat processing."""
 
-    has_decomposed_deploy_steps = False
-    """Whether the driver supports decomposed deploy steps.
-
-    Previously (since Rocky), drivers used a single 'deploy' deploy step on
-    the deploy interface. Some additional steps were added for the 'direct'
-    and 'iscsi' deploy interfaces in the Ussuri cycle, which means that
-    more of the deployment flow is driven by deploy steps.
-    """
-
     def __init__(self):
         self._client = _get_client()
-        if not self.has_decomposed_deploy_steps:
-            LOG.warning('%s does not support decomposed deploy steps. This '
-                        'is deprecated and will stop working in a future '
-                        'release', self.__class__.__name__)
-
-    def continue_deploy(self, task):
-        """Continues the deployment of baremetal node.
-
-        This method continues the deployment of the baremetal node after
-        the ramdisk have been booted.
-
-        :param task: a TaskManager instance
-        """
-
-    def deploy_has_started(self, task):
-        """Check if the deployment has started already.
-
-        :returns: True if the deploy has started, False otherwise.
-        """
-
-    def deploy_is_done(self, task):
-        """Check if the deployment is already completed.
-
-        :returns: True if the deployment is completed. False otherwise
-        """
-
-    def in_core_deploy_step(self, task):
-        """Check if we are in the deploy.deploy deploy step.
-
-        Assumes that we are in the DEPLOYWAIT state.
-
-        :param task: a TaskManager instance
-        :returns: True if the current deploy step is deploy.deploy.
-        """
-        step = task.node.deploy_step
-        return (step
-                and step['interface'] == 'deploy'
-                and step['step'] == 'deploy')
 
     def reboot_to_instance(self, task):
         """Method invoked after the deployment is completed.
@@ -495,10 +450,11 @@ class HeartbeatMixin(object):
         node = task.node
         if (node.provision_state in (states.CLEANING, states.CLEANWAIT)
                 and not CONF.conductor.allow_provisioning_in_maintenance):
-            LOG.error('Aborting cleaning for node %s, as it is in maintenance '
-                      'mode', node.uuid)
+            log_msg = ('Aborting cleaning for node %s, as it is in '
+                       'maintenance mode' % node.uuid)
             last_error = _('Cleaning aborted as node is in maintenance mode')
-            manager_utils.cleaning_error_handler(task, last_error)
+            manager_utils.cleaning_error_handler(task, log_msg,
+                                                 errmsg=last_error)
         elif (node.provision_state in (states.DEPLOYING, states.DEPLOYWAIT)
               and not CONF.conductor.allow_provisioning_in_maintenance):
             LOG.error('Aborting deployment for node %s, as it is in '
@@ -527,33 +483,14 @@ class HeartbeatMixin(object):
                 # booted and we need to collect in-band steps.
                 self.refresh_steps(task, 'deploy')
 
-            # NOTE(mgoddard): Only handle heartbeats during DEPLOYWAIT if we
-            # are currently in the core deploy.deploy step. Other deploy steps
-            # may cause the agent to boot, but we should not trigger deployment
-            # at that point if the driver is polling for completion of a step.
-            if (not self.has_decomposed_deploy_steps
-                    and self.in_core_deploy_step(task)):
-                msg = _('Failed checking if deploy is done')
-                # NOTE(mgoddard): support backwards compatibility for
-                # drivers which do not implement continue_deploy and
-                # reboot_to_instance as deploy steps.
-                if not self.deploy_has_started(task):
-                    msg = _('Node failed to deploy')
-                    self.continue_deploy(task)
-                elif self.deploy_is_done(task):
-                    msg = _('Node failed to move to active state')
-                    self.reboot_to_instance(task)
-                else:
-                    node.touch_provisioning()
-            else:
-                node.touch_provisioning()
-                # Check if the driver is polling for completion of
-                # a step, via the 'deployment_polling' flag.
-                polling = node.driver_internal_info.get(
-                    'deployment_polling', False)
-                if not polling:
-                    msg = _('Failed to process the next deploy step')
-                    self.process_next_step(task, 'deploy')
+            node.touch_provisioning()
+            # Check if the driver is polling for completion of
+            # a step, via the 'deployment_polling' flag.
+            polling = node.driver_internal_info.get(
+                'deployment_polling', False)
+            if not polling:
+                msg = _('Failed to process the next deploy step')
+                self.process_next_step(task, 'deploy')
         except Exception as e:
             last_error = _('%(msg)s. Error: %(exc)s') % {'msg': msg, 'exc': e}
             LOG.exception('Asynchronous exception for node %(node)s: %(err)s',
@@ -589,10 +526,11 @@ class HeartbeatMixin(object):
                     self.continue_cleaning(task)
         except Exception as e:
             last_error = _('%(msg)s. Error: %(exc)s') % {'msg': msg, 'exc': e}
-            LOG.exception('Asynchronous exception for node %(node)s: %(err)s',
-                          {'node': task.node.uuid, 'err': last_error})
+            log_msg = ('Asynchronous exception for node %(node)s: %(err)s' %
+                       {'node': task.node.uuid, 'err': last_error})
             if node.provision_state in (states.CLEANING, states.CLEANWAIT):
-                manager_utils.cleaning_error_handler(task, last_error)
+                manager_utils.cleaning_error_handler(task, log_msg,
+                                                     errmsg=last_error)
 
     def _heartbeat_rescue_wait(self, task):
         msg = _('Node failed to perform rescue operation')
@@ -607,12 +545,18 @@ class HeartbeatMixin(object):
                 manager_utils.rescuing_error_handler(task, last_error)
 
     @METRICS.timer('HeartbeatMixin.heartbeat')
-    def heartbeat(self, task, callback_url, agent_version):
+    def heartbeat(self, task, callback_url, agent_version,
+                  agent_verify_ca=None, agent_status=None,
+                  agent_status_message=None):
         """Process a heartbeat.
 
         :param task: task to work with.
         :param callback_url: agent HTTP API URL.
         :param agent_version: The version of the agent that is heartbeating
+        :param agent_verify_ca: TLS certificate for the agent.
+        :param agent_status: Status of the heartbeating agent
+        :param agent_status_message: Status message that describes the
+            agent_status
         """
         # NOTE(pas-ha) immediately skip the rest if nothing to do
         if (task.node.provision_state not in self.heartbeat_allowed_states
@@ -624,7 +568,7 @@ class HeartbeatMixin(object):
             return
 
         try:
-            task.upgrade_lock()
+            task.upgrade_lock(retry=False)
         except exception.NodeLocked:
             LOG.warning('Node %s is currently locked, skipping heartbeat '
                         'processing (will retry on the next heartbeat)',
@@ -641,6 +585,13 @@ class HeartbeatMixin(object):
         # datetime.datetime.strptime(var, "%Y-%m-%d %H:%M:%S.%f")
         driver_internal_info['agent_last_heartbeat'] = str(
             timeutils.utcnow().isoformat())
+        if agent_verify_ca:
+            driver_internal_info['agent_verify_ca'] = agent_verify_ca
+        if agent_status:
+            driver_internal_info['agent_status'] = agent_status
+        if agent_status_message:
+            driver_internal_info['agent_status_message'] = \
+                agent_status_message
         node.driver_internal_info = driver_internal_info
         node.save()
 
@@ -699,6 +650,13 @@ class AgentBaseMixin(object):
         """Whether agent boot is managed by ironic."""
         return True
 
+    def refresh_steps(self, task, step_type):
+        """Refresh the node's cached steps.
+
+        :param task: a TaskManager instance
+        :param step_type: "clean" or "deploy"
+        """
+
     @METRICS.timer('AgentBaseMixin.tear_down')
     @task_manager.require_exclusive_lock
     def tear_down(self, task):
@@ -759,8 +717,12 @@ class AgentBaseMixin(object):
             has an invalid value.
         :returns: states.CLEANWAIT to signify an asynchronous prepare
         """
-        return deploy_utils.prepare_inband_cleaning(
+        result = deploy_utils.prepare_inband_cleaning(
             task, manage_boot=self.should_manage_boot(task))
+        if result is None:
+            # Fast-track, ensure the steps are available.
+            self.refresh_steps(task, 'clean')
+        return result
 
     @METRICS.timer('AgentDeployMixin.tear_down_cleaning')
     def tear_down_cleaning(self, task):
@@ -894,11 +856,11 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
         call = getattr(self._client, 'get_%s_steps' % step_type)
         try:
             agent_result = call(node, task.ports).get('command_result', {})
-        except exception.AgentAPIError as exc:
-            if 'agent is busy' in str(exc):
-                LOG.debug('Agent is busy with a command, will refresh steps '
-                          'on the next heartbeat')
-                return
+        except exception.AgentInProgress as exc:
+            LOG.debug('Agent for node %(node)s is busy with a command, '
+                      'will refresh steps on the next heartbeat.',
+                      {'node': task.node.uuid})
+            return
 
             # TODO(dtantsur): change to just 'raise'
             if step_type == 'clean':
@@ -936,6 +898,8 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                                                     'step': step,
                                                     'type': step_type}))
 
+                if step_type == 'clean':
+                    step['requires_ramdisk'] = True
                 steps[step['interface']].append(step)
 
         # Save hardware manager version, steps, and date
@@ -1007,14 +971,14 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
             msg = (_('Could not continue cleaning on node '
                      '%(node)s: %(err)s.') %
                    {'node': node.uuid, 'err': e})
-            LOG.exception(msg)
-            return manager_utils.cleaning_error_handler(task, msg)
+            return manager_utils.cleaning_error_handler(task, msg,
+                                                        traceback=True)
         except exception.InstanceDeployFailure as e:
             msg = (_('Could not continue deployment on node '
                      '%(node)s: %(err)s.') %
                    {'node': node.uuid, 'err': e})
-            LOG.exception(msg)
-            return manager_utils.deploying_error_handler(task, msg)
+            return manager_utils.deploying_error_handler(task, msg,
+                                                         traceback=True)
 
         if manual_clean:
             # Don't restart manual cleaning if agent reboots to a new
@@ -1043,15 +1007,15 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                          '%(node)s after step %(step)s: %(err)s.') %
                        {'node': node.uuid, 'err': e,
                         'step': node.clean_step})
-                LOG.exception(msg)
-                return manager_utils.cleaning_error_handler(task, msg)
+                return manager_utils.cleaning_error_handler(task, msg,
+                                                            traceback=True)
             except exception.InstanceDeployFailure as e:
                 msg = (_('Could not restart deployment on node '
                          '%(node)s after step %(step)s: %(err)s.') %
                        {'node': node.uuid, 'err': e,
                         'step': node.deploy_step})
-                LOG.exception(msg)
-                return manager_utils.deploying_error_handler(task, msg)
+                return manager_utils.deploying_error_handler(task, msg,
+                                                             traceback=True)
 
         manager_utils.notify_conductor_resume_operation(task, step_type)
 
@@ -1105,7 +1069,6 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                     'err': agent_client.get_command_error(command),
                     'step': current_step,
                     'type': step_type})
-            LOG.error(msg)
             return _step_failure_handler(task, msg, step_type)
         # NOTE(dtantsur): VERSION_MISMATCH is a new alias for
         # CLEAN_VERSION_MISMATCH, remove the old one after IPA removes it.
@@ -1133,8 +1096,8 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                             'cls': e.__class__.__name__,
                             'step': current_step,
                             'type': step_type})
-                    LOG.exception(msg)
-                    return _step_failure_handler(task, msg, step_type)
+                    return _step_failure_handler(task, msg, step_type,
+                                                 traceback=True)
 
             if current_step.get('reboot_requested'):
                 _post_step_reboot(task, step_type)
@@ -1151,7 +1114,6 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                     'err': command.get('command_status'),
                     'step': current_step,
                     'type': step_type})
-            LOG.error(msg)
             return _step_failure_handler(task, msg, step_type)
 
     @METRICS.timer('AgentDeployMixin.tear_down_agent')
@@ -1162,14 +1124,15 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
 
         :param task: a TaskManager object containing the node
         """
-        wait = CONF.agent.post_deploy_get_power_state_retry_interval * 1000
+        wait = CONF.agent.post_deploy_get_power_state_retry_interval
         attempts = CONF.agent.post_deploy_get_power_state_retries + 1
 
-        @retrying.retry(
-            stop_max_attempt_number=attempts,
-            retry_on_result=lambda state: state != states.POWER_OFF,
-            wait_fixed=wait
-        )
+        @tenacity.retry(stop=tenacity.stop_after_attempt(attempts),
+                        retry=(tenacity.retry_if_result(
+                            lambda state: state != states.POWER_OFF)
+                            | tenacity.retry_if_exception_type(Exception)),
+                        wait=tenacity.wait_fixed(wait),
+                        reraise=True)
         def _wait_until_powered_off(task):
             return task.driver.power.get_power_state(task)
 
@@ -1213,7 +1176,7 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                                 'in at least %(timeout)d seconds. Forcing '
                                 'hard power off and proceeding.',
                                 {'node_uuid': node.uuid,
-                                 'timeout': (wait * (attempts - 1)) / 1000})
+                                 'timeout': (wait * (attempts - 1))})
                     manager_utils.node_power_action(task, states.POWER_OFF)
             else:
                 # Flush the file system prior to hard rebooting the node
@@ -1329,26 +1292,33 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
         # image's metadata (via Glance). Fall back to the driver internal
         # info in case it is not available (e.g. not set or there's no Glance).
         if software_raid:
-            image_source = node.instance_info.get('image_source')
-            try:
-                context = task.context
-                context.is_admin = True
-                glance = image_service.GlanceImageService(
-                    context=context)
-                image_info = glance.show(image_source)
-                image_properties = image_info.get('properties')
-                root_uuid = image_properties['rootfs_uuid']
-                LOG.debug('Got rootfs_uuid from Glance: %s '
-                          '(node %s)', root_uuid, node.uuid)
-            except Exception as e:
-                LOG.warning('Could not get \'rootfs_uuid\' property for '
-                            'image %(image)s from Glance for node %(node)s. '
-                            '%(cls)s: %(error)s.',
-                            {'image': image_source, 'node': node.uuid,
-                             'cls': e.__class__.__name__, 'error': e})
-                root_uuid = internal_info.get('root_uuid_or_disk_id')
-                LOG.debug('Got rootfs_uuid from driver internal info: '
-                          '%s (node %s)', root_uuid, node.uuid)
+            root_uuid = node.instance_info.get('image_rootfs_uuid')
+            if not root_uuid:
+                image_source = node.instance_info.get('image_source')
+                try:
+                    context = task.context
+                    # TODO(TheJulia): Uhh, is_admin likely needs to be
+                    # addressed in Xena as undesirable behavior may
+                    # result, or just outright break in an entirely
+                    # system scoped configuration.
+                    context.is_admin = True
+                    glance = image_service.GlanceImageService(
+                        context=context)
+                    image_info = glance.show(image_source)
+                    image_properties = image_info.get('properties')
+                    root_uuid = image_properties['rootfs_uuid']
+                    LOG.debug('Got rootfs_uuid from Glance: %s '
+                              '(node %s)', root_uuid, node.uuid)
+                except Exception as e:
+                    LOG.warning(
+                        'Could not get \'rootfs_uuid\' property for '
+                        'image %(image)s from Glance for node %(node)s. '
+                        '%(cls)s: %(error)s.',
+                        {'image': image_source, 'node': node.uuid,
+                         'cls': e.__class__.__name__, 'error': e})
+                    root_uuid = internal_info.get('root_uuid_or_disk_id')
+                    LOG.debug('Got rootfs_uuid from driver internal info: '
+                              '%s (node %s)', root_uuid, node.uuid)
 
         # For whole disk images it is not necessary that the root_uuid
         # be provided since the bootloaders on the disk will be used
