@@ -25,6 +25,7 @@ from ironic_lib import utils as ironic_utils
 from oslo_log import log as logging
 from oslo_utils import fileutils
 from oslo_utils import importutils
+from oslo_utils import strutils
 
 from ironic.common import boot_devices
 from ironic.common import exception
@@ -36,7 +37,6 @@ from ironic.common import utils
 from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
 from ironic.drivers.modules import boot_mode_utils
-from ironic.drivers.modules import deploy_utils
 
 ilo_client = importutils.try_import('proliantutils.ilo.client')
 ilo_error = importutils.try_import('proliantutils.exception')
@@ -56,7 +56,17 @@ REQUIRED_PROPERTIES = {
 OPTIONAL_PROPERTIES = {
     'client_port': _("port to be used for iLO operations. Optional."),
     'client_timeout': _("timeout (in seconds) for iLO operations. Optional."),
-    'ca_file': _("CA certificate file to validate iLO. Optional")
+    'ca_file': _("CA certificate file to validate iLO. This "
+                 "attibute is deprecated and will be removed in "
+                 "future release. Optional"),
+    'ilo_verify_ca': _("Either a Boolean value, a path to a CA_BUNDLE "
+                       "file or directory with certificates of trusted "
+                       "CAs. If set to True the driver will verify the "
+                       "host certificates; if False the driver will ignore "
+                       "verifying the SSL certificate. If it\'s a path the "
+                       "driver will use the specified certificate or one of "
+                       "the certificates in the directory. Defaults to True. "
+                       "Optional")
 }
 
 SNMP_PROPERTIES = {
@@ -125,6 +135,46 @@ SUPPORTED_BOOT_MODE_UEFI_ONLY = 'uefi only'
 SUPPORTED_BOOT_MODE_LEGACY_BIOS_AND_UEFI = 'legacy bios and uefi'
 """ Node supports both legacy BIOS and UEFI boot mode."""
 
+SUPPORTED_SECURITY_PARAMETERS = (
+    ['password_complexity', 'require_login_for_ilo_rbsu', 'ipmi_over_lan',
+     'secure_boot', 'require_host_authentication'])
+
+
+def validate_security_parameter_values(sec_param_info):
+    """Validate security parameter with valid values.
+
+    :param sec_param_info: dict object containing the security parameter info
+    :raises: MissingParameterValue, for missing fields (or values) in
+             security parameter info.
+    :raises: InvalidParameterValue, for unsupported security parameter
+    :returns: tuple of security param, ignore and enable parameters.
+    """
+    info = sec_param_info or {}
+
+    LOG.debug("Validating security param info: %s in progress", info)
+    param = info.get('param')
+
+    if not param:
+        msg = (_("Security parameter info: %(info)s is missing the "
+                 "required 'param' field.") % {'info': info})
+        LOG.error(msg)
+        raise exception.MissingParameterValue(msg)
+
+    if param not in SUPPORTED_SECURITY_PARAMETERS:
+        msg = (_("Security parameter '%(param)s' is not a valid parameter. "
+                 "Supported values are: %(supported_params)s") %
+               {'param': param, 'supported_params': (
+                ", ".join(SUPPORTED_SECURITY_PARAMETERS))})
+        LOG.error(msg)
+        raise exception.InvalidParameterValue(msg)
+
+    ignored = info.get('ignore', False)
+    ignored = strutils.bool_from_string(ignored, default=False)
+    enable = info.get('enable', True)
+    enable = strutils.bool_from_string(enable, default=True)
+
+    return param, enable, ignored
+
 
 def copy_image_to_web_server(source_file_path, destination):
     """Copies the given image to the http web server.
@@ -255,22 +305,8 @@ def parse_driver_info(node):
             "The following required iLO parameters are missing from the "
             "node's driver_info: %s") % missing_info)
 
-    not_integers = []
-    for param in OPTIONAL_PROPERTIES:
-        value = info.get(param, CONF.ilo.get(param))
-        if param == "client_port":
-            d_info[param] = utils.validate_network_port(value, param)
-        elif param == "ca_file":
-            if value and not os.path.isfile(value):
-                raise exception.InvalidParameterValue(_(
-                    '%(param)s "%(value)s" is not found.') %
-                    {'param': param, 'value': value})
-            d_info[param] = value
-        else:
-            try:
-                d_info[param] = int(value)
-            except ValueError:
-                not_integers.append(param)
+    optional_info = _parse_optional_driver_info(node)
+    d_info.update(optional_info)
 
     snmp_info = _parse_snmp_driver_info(info)
     if snmp_info:
@@ -284,11 +320,6 @@ def parse_driver_info(node):
             if param == "console_port":
                 d_info[param] = utils.validate_network_port(value, param)
 
-    if not_integers:
-        raise exception.InvalidParameterValue(_(
-            "The following iLO parameters from the node's driver_info "
-            "should be integers: %s") % not_integers)
-
     return d_info
 
 
@@ -335,6 +366,75 @@ def _parse_snmp_driver_info(info):
     return snmp_info
 
 
+def _parse_optional_driver_info(node):
+    """Parses the optional driver_info parameters.
+
+    :param node: an ironic Node object.
+    :returns: a dictionary containing information of optional properties.
+    :raises: InvalidParameterValue if any parameters are incorrect
+    """
+    info = node.driver_info
+    optional_info = {}
+    not_integers = []
+
+    for param in OPTIONAL_PROPERTIES:
+        if param != 'ilo_verify_ca':
+            value = info.get(param, CONF.ilo.get(param))
+
+        if param == "client_port":
+            optional_info[param] = utils.validate_network_port(value, param)
+        elif param == "client_timeout":
+            try:
+                optional_info[param] = int(value)
+            except ValueError:
+                not_integers.append(param)
+
+    if not_integers:
+        raise exception.InvalidParameterValue(_(
+            "The following iLO parameters from the node's driver_info "
+            "should be integers: %s") % not_integers)
+
+    ca_file_value = info.get('ca_file', CONF.ilo.get('ca_file'))
+    verify_ca = info.get('ilo_verify_ca', CONF.ilo.get('verify_ca'))
+
+    if ca_file_value:
+        LOG.warning("The `driver_info/ca_file` parameter is deprecated "
+                    "in favor of `driver_info/ilo_verify_ca` parameter. "
+                    "Please use `driver_info/ilo_verify_ca` instead of "
+                    "`driver_info/ca_file` in node %(node)s "
+                    "configuration.", {'node': node.uuid})
+
+        if not os.path.isfile(ca_file_value):
+            raise exception.InvalidParameterValue(_(
+                'ca_file "%(value)s" is not found.') %
+                {'value': ca_file_value})
+        optional_info['verify_ca'] = ca_file_value
+    else:
+        # Check if ilo_verify_ca is a Boolean or a file/directory
+        # in the file-system
+        if isinstance(verify_ca, str):
+            if not os.path.isdir(verify_ca) and not os.path.isfile(verify_ca):
+                try:
+                    verify_ca = strutils.bool_from_string(verify_ca,
+                                                          strict=True)
+                except ValueError:
+                    raise exception.InvalidParameterValue(
+                        _('Invalid value type set in driver_info/'
+                          'ilo_verify_ca on node %(node)s. '
+                          'The value should be a Boolean or the path '
+                          'to a file/directory, not "%(value)s"'
+                          ) % {'value': verify_ca, 'node': node.uuid})
+        elif not isinstance(verify_ca, bool):
+            raise exception.InvalidParameterValue(
+                _('Invalid value type set in driver_info/ilo_verify_ca '
+                  'on node %(node)s. The value should be a Boolean '
+                  'or the path to a file/directory, not "%(value)s"'
+                  ) % {'value': verify_ca, 'node': node.uuid})
+        optional_info['verify_ca'] = verify_ca
+
+    return optional_info
+
+
 def get_ilo_object(node):
     """Gets an IloClient object from proliantutils library.
 
@@ -369,7 +469,7 @@ def get_ilo_object(node):
                                       driver_info['ilo_password'],
                                       driver_info['client_timeout'],
                                       driver_info['client_port'],
-                                      cacert=driver_info.get('ca_file'),
+                                      cacert=driver_info['verify_ca'],
                                       snmp_credentials=info)
     return ilo_object
 
@@ -484,10 +584,11 @@ def set_boot_mode(node, boot_mode):
     except ilo_error.IloCommandNotSupportedError:
         p_boot_mode = DEFAULT_BOOT_MODE
 
-    if BOOT_MODE_ILO_TO_GENERIC[p_boot_mode.lower()] == boot_mode:
-        LOG.info("Node %(uuid)s pending boot mode is %(boot_mode)s.",
-                 {'uuid': node.uuid, 'boot_mode': boot_mode})
-        return
+    if p_boot_mode:
+        if BOOT_MODE_ILO_TO_GENERIC[p_boot_mode.lower()] == boot_mode:
+            LOG.info("Node %(uuid)s pending boot mode is %(boot_mode)s.",
+                     {'uuid': node.uuid, 'boot_mode': boot_mode})
+            return
 
     try:
         ilo_object.set_pending_boot_mode(
@@ -536,9 +637,9 @@ def update_boot_mode(task):
     # No boot mode found. Check if default_boot_mode is defined
     if not boot_mode and (CONF.ilo.default_boot_mode in ['bios', 'uefi']):
         boot_mode = CONF.ilo.default_boot_mode
-        instance_info = node.instance_info
-        instance_info['deploy_boot_mode'] = boot_mode
-        node.instance_info = instance_info
+        driver_internal_info = node.driver_internal_info
+        driver_internal_info['deploy_boot_mode'] = boot_mode
+        node.driver_internal_info = driver_internal_info
         node.save()
 
     # Boot mode is computed, setting it for the deploy
@@ -578,13 +679,13 @@ def update_boot_mode(task):
                   "as pending boot mode is unknown.",
                   {'uuid': node.uuid, 'boot_mode': boot_mode})
 
-    instance_info = node.instance_info
-    instance_info['deploy_boot_mode'] = boot_mode
-    node.instance_info = instance_info
+    driver_internal_info = node.driver_internal_info
+    driver_internal_info['deploy_boot_mode'] = boot_mode
+    node.driver_internal_info = driver_internal_info
     node.save()
 
 
-def setup_vmedia(task, iso, ramdisk_options):
+def setup_vmedia(task, iso, ramdisk_options=None):
     """Attaches virtual media and sets it as boot device.
 
     This method attaches the given bootable ISO as virtual media, prepares the
@@ -769,28 +870,6 @@ def set_secure_boot_mode(task, flag):
                                           error=ilo_exception)
 
 
-def update_secure_boot_mode(task, mode):
-    """Changes secure boot mode for next boot on the node.
-
-    This method changes secure boot mode on the node for next boot. It changes
-    the secure boot mode setting on node only if the deploy has requested for
-    the secure boot.
-    During deploy, this method is used to enable secure boot on the node by
-    passing 'mode' as 'True'.
-    During teardown, this method is used to disable secure boot on the node by
-    passing 'mode' as 'False'.
-
-    :param task: a TaskManager instance containing the node to act on.
-    :param mode: Boolean value requesting the next state for secure boot
-    :raises: IloOperationNotSupported, if operation is not supported on iLO
-    :raises: IloOperationError, if some operation on iLO failed.
-    """
-    if deploy_utils.is_secure_boot_requested(task.node):
-        set_secure_boot_mode(task, mode)
-        LOG.info('Changed secure boot to %(mode)s for node %(node)s',
-                 {'mode': mode, 'node': task.node.uuid})
-
-
 def remove_single_or_list_of_files(file_location):
     """Removes (deletes) the file or list of files.
 
@@ -861,3 +940,190 @@ def get_server_post_state(node):
     except ilo_error.IloError as ilo_exception:
         raise exception.IloOperationError(operation=operation,
                                           error=ilo_exception)
+
+
+def _get_certificate_file_list(cert_file_list):
+    """Get the list of certificates to use.
+
+    :param cert_file_list: certificates file list.
+    :returns: cert_file_list if it's not empty. If empty or None,
+        returns the list of path configured for "webserver_verify_ca"
+        configuration option if the path exists. If the path does not
+        exist, returns empty list.
+    :raises: InvalidParameterValue if argument provided is other than
+        a list.
+    """
+
+    cfl = cert_file_list
+
+    if not cfl:
+        try:
+            verify = strutils.bool_from_string(CONF.webserver_verify_ca,
+                                               strict=True)
+        except ValueError:
+            verify = CONF.webserver_verify_ca
+
+        if isinstance(verify, bool):
+            return []
+
+        if not os.path.exists(verify):
+            LOG.error("Path to the certificate file %(path)s "
+                      "does not exist.", {'path': verify})
+            return []
+
+        cfl = [verify]
+
+    if not isinstance(cfl, list):
+        raise exception.InvalidParameterValue(_(
+            'List of files is expected whereas "%(atype)s" type '
+            'is provided.') % {'atype': type(cfl)})
+
+    return cfl
+
+
+def add_certificates(task, cert_file_list=None):
+    """Adds certificates to the node.
+
+    Adds certificates to the node based on the driver info
+    provided.
+
+    :param task: a TaskManager instance containing the node to act on.
+    :param cert_file_list: List of certificates to be added to the node.
+        If None, certificates from path configured in 'webserver_verify_ca'
+        will be added to the node.
+    :raises: IloOperationError on an error from IloClient library.
+    :raises: IloOperationNotSupported if retrieving post state is not
+        supported on the server.
+    :raises: InvalidParameterValue, if any of the required parameters are
+            invalid.
+    """
+
+    node = task.node
+    ilo_object = get_ilo_object(node)
+    d_info = node.driver_info
+
+    export_certs = d_info.get('ilo_add_certificates', True)
+
+    if export_certs is None:
+        export_certs = True
+    else:
+        try:
+            export_certs = strutils.bool_from_string(export_certs,
+                                                     strict=True)
+        except ValueError:
+            raise exception.InvalidParameterValue(
+                _('Invalid value type set in driver_info/'
+                  'ilo_add_certificates on node %(node)s. '
+                  'The value should be a Boolean '
+                  ' not "%(value)s"'
+                  ) % {'value': export_certs, 'node': node.uuid})
+
+    if not export_certs:
+        LOG.info("Adding of certificates to the node %(node)s is not "
+                 "requested. Assuming required certificates are available "
+                 "on the node.", {'node': node.uuid})
+        return
+
+    cfl = _get_certificate_file_list(cert_file_list)
+
+    if not cfl:
+        LOG.debug("Not adding any certificate to the node %(node)s "
+                  "as no certificates are provided", {'node': node.uuid})
+        return
+
+    try:
+        # NOTE(vmud213): Add the certificates to the node which are
+        # eventually being used for TLS verification by the node before
+        # downloading the deploy/instance images during HTTPS boot from
+        # URL.
+        operation = (_("Add certificates to %(node)s from paths "
+                       "%(cpath)s.") % {'cpath': cfl, 'node': node.uuid})
+
+        ilo_object.add_tls_certificate(cfl)
+
+        LOG.info("Successfully added certificates to the node %(node)s from "
+                 "paths %(cpath)s.", {'cpath': cfl, 'node': node.uuid})
+    except ilo_error.IloCommandNotSupportedInBiosError as ilo_exception:
+        raise exception.IloOperationNotSupported(operation=operation,
+                                                 error=ilo_exception)
+    except ilo_error.IloError as ilo_exception:
+        raise exception.IloOperationError(operation=operation,
+                                          error=ilo_exception)
+
+
+def clear_certificates(task, cert_file_list=None):
+    """Clears any certificates added to the node.
+
+    Clears the certificates added to the node as part of any Ironic
+    operation
+
+    :param task: a TaskManager instance containing the node to act on.
+    :param cert_file_list: List of certificates to be removed from node.
+        If None, all the certificates present on the node will be removed.
+    :raises: IloOperationError on an error from IloClient library.
+    :raises: IloOperationNotSupported if retrieving post state is not
+        supported on the server.
+    """
+
+    node = task.node
+    operation = (_("Clearing certificates from node %(node)s.") %
+                 {'node': node.uuid})
+
+    try:
+        ilo_object = get_ilo_object(node)
+        ilo_object.remove_tls_certificate(cert_file_list)
+    except ilo_error.IloCommandNotSupportedInBiosError as ilo_exception:
+        raise exception.IloOperationNotSupported(operation=operation,
+                                                 error=ilo_exception)
+    except ilo_error.IloError as ilo_exception:
+        raise exception.IloOperationError(operation=operation,
+                                          error=ilo_exception)
+    LOG.info("Cleared TLS certificates from the node %(node)s "
+             "successfully from paths %(cpath)s.",
+             {'node': node.uuid, 'cpath': cert_file_list})
+
+
+def setup_uefi_https(task, iso, persistent=False):
+    """Sets up system to boot from UEFIHTTP boot device.
+
+    Sets the one-time/persistent boot device to UEFIHTTP based
+    on the argument supplied.
+
+    :param task: a TaskManager instance containing the node to act on.
+    :param iso: ISO URL to be set to boot from.
+    :param persistent: Indicates whether the system should be set to boot
+        from the given device one-time or each time.
+    :raises: IloOperationError on an error from IloClient library.
+    :raises: IloOperationNotSupported if retrieving post state is not
+        supported on the server.
+    """
+    node = task.node
+    ilo_object = get_ilo_object(node)
+    scheme = urlparse.urlparse(iso).scheme.lower()
+
+    operation = (_("Setting up node %(node)s to boot from URL %(iso)s.") %
+                 {'iso': iso, 'node': node.uuid})
+
+    if scheme != 'https':
+        msg = (_('Error setting up node %(node)s to boot from '
+                 'URL %(iso)s. A secure URL is expected that is exposed '
+                 'over HTTPS.') %
+               {'node': node.uuid, 'iso': iso})
+        raise exception.IloOperationNotSupported(operation=operation,
+                                                 error=msg)
+
+    try:
+        ilo_object.set_http_boot_url(iso)
+        LOG.info("Set the node %(node)s to boot from URL %(iso)s "
+                 "successfully.", {'node': node.uuid, 'iso': iso})
+        if not persistent:
+            ilo_object.set_one_time_boot('UEFIHTTP')
+        else:
+            ilo_object.update_persistent_boot(['UEFIHTTP'])
+
+    except ilo_error.IloCommandNotSupportedInBiosError as ilo_exception:
+        raise exception.IloOperationNotSupported(operation=operation,
+                                                 error=ilo_exception)
+    except ilo_error.IloError as ilo_exception:
+        raise exception.IloOperationError(operation=operation,
+                                          error=ilo_exception)
