@@ -33,61 +33,60 @@ from ironic.common import utils
 from ironic.conf import CONF
 from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
+from ironic.drivers import utils as driver_utils
 
 LOG = log.getLogger(__name__)
 
 
 class ImageHandler(object):
 
-    _SWIFT_MAP = {
-        "redfish": {
-            "swift_enabled": CONF.redfish.use_swift,
-            "container": CONF.redfish.swift_container,
-            "timeout": CONF.redfish.swift_object_expiry_timeout,
-            "image_subdir": "redfish",
-            "file_permission": CONF.redfish.file_permission,
-            "kernel_params": CONF.redfish.kernel_append_params
-        },
-        "idrac": {
-            "swift_enabled": CONF.redfish.use_swift,
-            "container": CONF.redfish.swift_container,
-            "timeout": CONF.redfish.swift_object_expiry_timeout,
-            "image_subdir": "redfish",
-            "file_permission": CONF.redfish.file_permission,
-            "kernel_params": CONF.redfish.kernel_append_params
-        },
-        "ilo5": {
-            "swift_enabled": not CONF.ilo.use_web_server_for_images,
-            "container": CONF.ilo.swift_ilo_container,
-            "timeout": CONF.ilo.swift_object_expiry_timeout,
-            "image_subdir": "ilo",
-            "file_permission": CONF.ilo.file_permission,
-            "kernel_params": CONF.pxe.pxe_append_params
-        },
-        "ilo": {
-            "swift_enabled": not CONF.ilo.use_web_server_for_images,
-            "container": CONF.ilo.swift_ilo_container,
-            "timeout": CONF.ilo.swift_object_expiry_timeout,
-            "image_subdir": "ilo",
-            "file_permission": CONF.ilo.file_permission,
-            "kernel_params": CONF.pxe.pxe_append_params
-        },
-    }
-
     def __init__(self, driver):
-        self._driver = driver
-        self._container = self._SWIFT_MAP[driver].get("container")
-        self._timeout = self._SWIFT_MAP[driver].get("timeout")
-        self._image_subdir = self._SWIFT_MAP[driver].get("image_subdir")
-        self._file_permission = self._SWIFT_MAP[driver].get("file_permission")
-        # To get the kernel parameters
-        self.kernel_params = self._SWIFT_MAP[driver].get("kernel_params")
+        self.update_driver_config(driver)
 
-    def _is_swift_enabled(self):
-        try:
-            return self._SWIFT_MAP[self._driver].get("swift_enabled")
-        except KeyError:
-            return False
+    def update_driver_config(self, driver):
+        _SWIFT_MAP = {
+            "redfish": {
+                "swift_enabled": CONF.redfish.use_swift,
+                "container": CONF.redfish.swift_container,
+                "timeout": CONF.redfish.swift_object_expiry_timeout,
+                "image_subdir": "redfish",
+                "file_permission": CONF.redfish.file_permission,
+                "kernel_params": CONF.redfish.kernel_append_params
+            },
+            "idrac": {
+                "swift_enabled": CONF.redfish.use_swift,
+                "container": CONF.redfish.swift_container,
+                "timeout": CONF.redfish.swift_object_expiry_timeout,
+                "image_subdir": "redfish",
+                "file_permission": CONF.redfish.file_permission,
+                "kernel_params": CONF.redfish.kernel_append_params
+            },
+            "ilo5": {
+                "swift_enabled": not CONF.ilo.use_web_server_for_images,
+                "container": CONF.ilo.swift_ilo_container,
+                "timeout": CONF.ilo.swift_object_expiry_timeout,
+                "image_subdir": "ilo",
+                "file_permission": CONF.ilo.file_permission,
+                "kernel_params": CONF.ilo.kernel_append_params
+            },
+            "ilo": {
+                "swift_enabled": not CONF.ilo.use_web_server_for_images,
+                "container": CONF.ilo.swift_ilo_container,
+                "timeout": CONF.ilo.swift_object_expiry_timeout,
+                "image_subdir": "ilo",
+                "file_permission": CONF.ilo.file_permission,
+                "kernel_params": CONF.ilo.kernel_append_params
+            },
+        }
+
+        self._driver = driver
+        self.swift_enabled = _SWIFT_MAP[driver].get("swift_enabled")
+        self._container = _SWIFT_MAP[driver].get("container")
+        self._timeout = _SWIFT_MAP[driver].get("timeout")
+        self._image_subdir = _SWIFT_MAP[driver].get("image_subdir")
+        self._file_permission = _SWIFT_MAP[driver].get("file_permission")
+        # To get the kernel parameters
+        self.kernel_params = _SWIFT_MAP[driver].get("kernel_params")
 
     def unpublish_image(self, object_name):
         """Withdraw the image previously made downloadable.
@@ -98,7 +97,7 @@ class ImageHandler(object):
 
         :param object_name: name of the published file (optional)
         """
-        if self._is_swift_enabled():
+        if self.swift_enabled:
             container = self._container
 
             swift_api = swift.SwiftAPI()
@@ -179,7 +178,7 @@ class ImageHandler(object):
         :return: a URL to download published file
         """
 
-        if self._is_swift_enabled():
+        if self.swift_enabled:
             container = self._container
             timeout = self._timeout
 
@@ -216,8 +215,8 @@ class ImageHandler(object):
                 shutil.copyfile(image_file, published_file)
                 os.chmod(published_file, self._file_permission)
 
-            image_url = os.path.join(
-                CONF.deploy.http_url, self._image_subdir, object_name)
+            http_url = CONF.deploy.external_http_url or CONF.deploy.http_url
+            image_url = os.path.join(http_url, self._image_subdir, object_name)
 
         image_url = self._append_filename_param(
             image_url, os.path.basename(image_file))
@@ -246,6 +245,16 @@ def cleanup_iso_image(task):
                                           suffix='.iso')
 
 
+def override_api_url(params):
+    if not CONF.deploy.external_callback_url:
+        return params
+
+    params = params or {}
+    params[deploy_utils.IPA_URL_PARAM_NAME] = \
+        CONF.deploy.external_callback_url.rstrip('/')
+    return params
+
+
 def prepare_floppy_image(task, params=None):
     """Prepares the floppy image for passing the parameters.
 
@@ -266,6 +275,7 @@ def prepare_floppy_image(task, params=None):
     :returns: image URL for the floppy image.
     """
     object_name = _get_name(task.node, prefix='image')
+    params = override_api_url(params)
 
     LOG.debug("Trying to create floppy image for node "
               "%(node)s", {'node': task.node.uuid})
@@ -395,7 +405,7 @@ def _prepare_iso_image(task, kernel_href, ramdisk_href,
     :param kernel_href: URL or Glance UUID of the kernel to use
     :param ramdisk_href: URL or Glance UUID of the ramdisk to use
     :param bootloader_href: URL or Glance UUID of the EFI bootloader
-         image to use when creating UEFI bootbable ISO
+         image to use when creating UEFI bootable ISO
     :param root_uuid: optional uuid of the root partition.
     :param params: a dictionary containing 'parameter name'->'value'
         mapping to be passed to kernel command line.
@@ -414,8 +424,16 @@ def _prepare_iso_image(task, kernel_href, ramdisk_href,
             "building ISO, or explicit ISO for %(node)s") %
             {'node': task.node.uuid})
 
+    # NOTE(rpittau): if base_iso is defined as http address, we just access
+    # it directly.
+    if base_iso and CONF.deploy.ramdisk_image_download_source == 'http':
+        if base_iso.startswith(('http://', 'https://')):
+            return base_iso
+        LOG.debug("ramdisk_image_download_source set to http but "
+                  "boot_iso is not an HTTP URL: %(boot_iso)s",
+                  {"boot_iso": base_iso})
+
     img_handler = ImageHandler(task.node.driver)
-    k_param = img_handler.kernel_params
 
     i_info = task.node.instance_info
 
@@ -429,24 +447,34 @@ def _prepare_iso_image(task, kernel_href, ramdisk_href,
             kernel_params = None
 
     else:
-        kernel_params = i_info.get('kernel_append_params', k_param)
+        kernel_params = driver_utils.get_kernel_append_params(
+            task.node, default=img_handler.kernel_params)
 
     if params and not base_iso:
         kernel_params = ' '.join(
             (kernel_params, ' '.join(
-                '%s=%s' % kv for kv in params.items())))
+                ('%s=%s' % kv) if kv[1] is not None else kv[0]
+                for kv in params.items())))
 
     boot_mode = boot_mode_utils.get_boot_mode(task.node)
 
-    LOG.debug("Trying to create %(boot_mode)s ISO image for node %(node)s "
-              "with kernel %(kernel_href)s, ramdisk %(ramdisk_href)s, "
-              "bootloader %(bootloader_href)s and kernel params %(params)s"
-              "", {'node': task.node.uuid,
-                   'boot_mode': boot_mode,
-                   'kernel_href': kernel_href,
-                   'ramdisk_href': ramdisk_href,
-                   'bootloader_href': bootloader_href,
-                   'params': kernel_params})
+    if base_iso:
+        # TODO(dtantsur): fix this limitation eventually (see
+        # images.create_boot_iso).
+        LOG.debug('Using pre-built %(boot_mode)s ISO %(iso)s for node '
+                  '%(node)s, custom configuration will not be available',
+                  {'boot_mode': boot_mode, 'node': task.node.uuid,
+                   'iso': base_iso})
+    else:
+        LOG.debug("Trying to create %(boot_mode)s ISO image for node %(node)s "
+                  "with kernel %(kernel_href)s, ramdisk %(ramdisk_href)s, "
+                  "bootloader %(bootloader_href)s and kernel params %(params)s"
+                  "", {'node': task.node.uuid,
+                       'boot_mode': boot_mode,
+                       'kernel_href': kernel_href,
+                       'ramdisk_href': ramdisk_href,
+                       'bootloader_href': bootloader_href,
+                       'params': kernel_params})
 
     with tempfile.NamedTemporaryFile(
             dir=CONF.tempdir, suffix='.iso') as boot_fileobj:
@@ -481,7 +509,8 @@ def _find_param(param_str, param_dict):
     for param_key in param_dict:
         if param_str in param_key:
             val = param_dict.get(param_key)
-    return val
+            if val is not None:
+                return val
 
 
 _TLS_REMOTE_FILE = 'etc/ironic-python-agent/ironic.crt'
@@ -518,18 +547,22 @@ def prepare_deploy_iso(task, params, mode, d_info):
 
     kernel_str = '%s_kernel' % mode
     ramdisk_str = '%s_ramdisk' % mode
+    iso_str = '%s_iso' % mode
     bootloader_str = 'bootloader'
 
     kernel_href = _find_param(kernel_str, d_info)
     ramdisk_href = _find_param(ramdisk_str, d_info)
+    iso_href = _find_param(iso_str, d_info)
     bootloader_href = _find_param(bootloader_str, d_info)
 
+    params = override_api_url(params)
+
     # TODO(TheJulia): At some point we should support something like
     # boot_iso for the deploy interface, perhaps when we support config
     # injection.
     prepare_iso_image = functools.partial(
         _prepare_iso_image, task, kernel_href, ramdisk_href,
-        bootloader_href=bootloader_href, params=params)
+        bootloader_href=bootloader_href, params=params, base_iso=iso_href)
 
     inject_files = {}
     if CONF.agent.api_ca_file:
