@@ -15,7 +15,7 @@
 
 from oslo_log import log
 from oslo_utils import importutils
-import retrying
+import tenacity
 
 from ironic.common import boot_devices
 from ironic.common import exception
@@ -35,16 +35,18 @@ LOG = log.getLogger(__name__)
 
 REQUIRED_PROPERTIES = {
     'deploy_kernel': _("URL or Glance UUID of the deployment kernel. "
-                       "Required."),
-    'deploy_ramdisk': _("URL or Glance UUID of the ramdisk that is "
-                        "mounted at boot time. Required.")
+                       "Required if deploy_iso is not set."),
+    'deploy_ramdisk': _("URL or Glance UUID of the ramdisk that is mounted at "
+                        "boot time. Required if deploy_iso is not set."),
+    'deploy_iso': _("URL or Glance UUID of the deployment ISO to use. "
+                    "Required if deploy_kernel/deploy_ramdisk are not set.")
 }
 
 OPTIONAL_PROPERTIES = {
-    'config_via_floppy': _("Boolean value to indicate whether or not the "
-                           "driver should use virtual media Floppy device "
-                           "for passing configuration information to the "
-                           "ramdisk. Defaults to False. Optional."),
+    'config_via_removable': _("Boolean value to indicate whether or not the "
+                              "driver should use virtual media USB or floppy "
+                              "device for passing configuration information "
+                              "to the ramdisk. Defaults to False. Optional."),
     'kernel_append_params': _("Additional kernel parameters to pass down to "
                               "instance kernel. These parameters can be "
                               "consumed by the kernel or by the applications "
@@ -56,16 +58,19 @@ OPTIONAL_PROPERTIES = {
                     "image containing EFI boot loader. This image will be "
                     "used by ironic when building UEFI-bootable ISO "
                     "out of kernel and ramdisk. Required for UEFI "
-                    "boot from partition images."),
+                    "when deploy_iso is not provided."),
 
 }
 
 RESCUE_PROPERTIES = {
-    'rescue_kernel': _('URL or Glance UUID of the rescue kernel. This value '
-                       'is required for rescue mode.'),
+    'rescue_kernel': _('URL or Glance UUID of the rescue kernel. Required for '
+                       'rescue mode if rescue_iso is not set.'),
     'rescue_ramdisk': _('URL or Glance UUID of the rescue ramdisk with agent '
-                        'that is used at node rescue time. This value is '
-                        'required for rescue mode.'),
+                        'that is used at node rescue time. Required for '
+                        'rescue mode if rescue_iso is not set.'),
+    'rescue_iso': _("URL or Glance UUID of the rescue ISO to use. Required "
+                    "for rescue mode if rescue_kernel/rescue_ramdisk are "
+                    "not set.")
 }
 
 COMMON_PROPERTIES = REQUIRED_PROPERTIES.copy()
@@ -73,11 +78,6 @@ COMMON_PROPERTIES.update(driver_utils.OPTIONAL_PROPERTIES)
 COMMON_PROPERTIES.update(OPTIONAL_PROPERTIES)
 COMMON_PROPERTIES.update(RESCUE_PROPERTIES)
 
-KERNEL_RAMDISK_LABELS = {
-    'deploy': REQUIRED_PROPERTIES,
-    'rescue': RESCUE_PROPERTIES
-}
-
 IMAGE_SUBDIR = 'redfish'
 
 sushy = importutils.try_import('sushy')
@@ -100,28 +100,38 @@ def _parse_driver_info(node):
     d_info = node.driver_info
 
     mode = deploy_utils.rescue_or_deploy_mode(node)
-    params_to_check = KERNEL_RAMDISK_LABELS[mode]
+    iso_param = f'{mode}_iso'
+    iso_ref = driver_utils.get_agent_iso(node, deprecated_prefix='redfish',
+                                         mode=mode)
+    if iso_ref is not None:
+        deploy_info = {iso_param: iso_ref}
+        can_config = False
+    else:
+        # There was never a deprecated prefix for kernel/ramdisk
+        deploy_info = driver_utils.get_agent_kernel_ramdisk(node, mode)
 
-    deploy_info = {option: d_info.get(option)
-                   for option in params_to_check}
+        error_msg = _("Error validating Redfish virtual media. Some "
+                      "parameters were missing in node's driver_info")
 
-    if not any(deploy_info.values()):
-        # NOTE(dtantsur): avoid situation when e.g. deploy_kernel comes
-        # from driver_info but deploy_ramdisk comes from configuration,
-        # since it's a sign of a potential operator's mistake.
-        deploy_info = {k: getattr(CONF.conductor, k)
-                       for k in params_to_check}
-
-    error_msg = _("Error validating Redfish virtual media. Some "
-                  "parameters were missing in node's driver_info")
-
-    deploy_utils.check_for_missing_params(deploy_info, error_msg)
+        deploy_utils.check_for_missing_params(deploy_info, error_msg)
+        can_config = True
 
     deploy_info.update(
         {option: d_info.get(option, getattr(CONF.conductor, option, None))
          for option in OPTIONAL_PROPERTIES})
 
+    if (d_info.get('config_via_removable') is None
+            and d_info.get('config_via_floppy') is not None):
+        LOG.warning('The config_via_floppy driver_info option is deprecated, '
+                    'use config_via_removable for node %s', node.uuid)
+        deploy_info['config_via_removable'] = d_info['config_via_floppy']
+
     deploy_info.update(redfish_utils.parse_driver_info(node))
+    # Configuration can be provided in one of two cases:
+    # 1) A removable disk is requested.
+    # 2) An ISO is built from a kernel/initramfs pair.
+    deploy_info['can_provide_config'] = \
+        deploy_info.get('config_via_removable') or can_config
 
     return deploy_info
 
@@ -155,8 +165,10 @@ def _test_retry(exception):
     return False
 
 
-@retrying.retry(wait_fixed=3000, stop_max_attempt_number=3,
-                retry_on_exception=_test_retry)
+@tenacity.retry(retry=tenacity.retry_if_exception(_test_retry),
+                stop=tenacity.stop_after_attempt(3),
+                wait=tenacity.wait_fixed(3),
+                reraise=True)
 def _insert_vmedia(task, managers, boot_url, boot_device):
     """Insert bootable ISO image into virtual CD or DVD
 
@@ -210,9 +222,11 @@ def _eject_vmedia(task, managers, boot_device=None):
     :param boot_device: sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
         `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY` or `None` to
         eject everything (default).
+    :return: True if any device was ejected, else False
     :raises: InvalidParameterValue, if no suitable virtual CD or DVD is
         found on the node.
     """
+    found = False
     for manager in managers:
         for v_media in manager.virtual_media.get_members():
             if boot_device and boot_device not in v_media.media_types:
@@ -222,12 +236,14 @@ def _eject_vmedia(task, managers, boot_device=None):
 
             if inserted:
                 v_media.eject_media()
+                found = True
 
             LOG.info("Boot media is%(already)s ejected from "
                      "%(boot_device)s for node %(node)s"
                      "", {'node': task.node.uuid,
                           'already': '' if inserted else ' already',
                           'boot_device': v_media.name})
+    return found
 
 
 def eject_vmedia(task, boot_device=None):
@@ -241,20 +257,40 @@ def eject_vmedia(task, boot_device=None):
         found on the node.
     """
     system = redfish_utils.get_system(task.node)
-    _eject_vmedia(task, system.managers, boot_device=boot_device)
+    if _eject_vmedia(task, system.managers, boot_device=boot_device):
+        LOG.debug('Cleaning up unused files after ejecting %(dev)s for node '
+                  '%(node)s', {'dev': boot_device or 'all devices',
+                               'node': task.node.uuid})
+        if (boot_device is None
+                or boot_device == sushy.VIRTUAL_MEDIA_USBSTICK):
+            image_utils.cleanup_disk_image(task, prefix='configdrive')
+        if boot_device is None or boot_device == sushy.VIRTUAL_MEDIA_CD:
+            image_utils.cleanup_iso_image(task)
 
 
-def _has_vmedia_device(managers, boot_device):
+def _has_vmedia_device(managers, boot_device, inserted=None):
     """Indicate if device exists at any of the managers
 
     :param managers: A list of System managers.
-    :param boot_device: sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
-        `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY`.
+    :param boot_device: One or more sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
+        `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY`. Several devices are
+        checked in the given order.
+    :param inserted: If not None, only return a device with a matching
+        inserted status.
+    :return: The device that could be found or False.
     """
-    for manager in managers:
-        for v_media in manager.virtual_media.get_members():
-            if boot_device in v_media.media_types:
-                return True
+    if isinstance(boot_device, str):
+        boot_device = [boot_device]
+
+    for dev in boot_device:
+        for manager in managers:
+            for v_media in manager.virtual_media.get_members():
+                if dev not in v_media.media_types:
+                    continue
+                if (inserted is not None
+                        and bool(v_media.inserted) is not inserted):
+                    continue
+                return dev
     return False
 
 
@@ -346,6 +382,8 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         node = task.node
 
         _parse_driver_info(node)
+        # Issue the deprecation warning if needed
+        driver_utils.get_agent_iso(node, deprecated_prefix='redfish')
 
     def _validate_instance_info(self, task):
         """Validate instance image information for the task's node.
@@ -360,6 +398,13 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         """
         node = task.node
 
+        # NOTE(dtantsur): if we're are writing an image with local boot
+        # the boot interface does not care about image parameters and
+        # must not validate them.
+        if (not task.driver.storage.should_write_image(task)
+                or deploy_utils.get_boot_option(node) == 'local'):
+            return
+
         d_info = _parse_deploy_info(node)
 
         if node.driver_internal_info.get('is_whole_disk_image'):
@@ -400,9 +445,7 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         """
         self._validate_vendor(task)
         self._validate_driver_info(task)
-
-        if task.driver.storage.should_write_image(task):
-            self._validate_instance_info(task)
+        self._validate_instance_info(task)
 
     def validate_inspection(self, task):
         """Validate that the node has required properties for inspection.
@@ -447,48 +490,66 @@ class RedfishVirtualMediaBoot(base.BootInterface):
                                         states.INSPECTING):
             return
 
+        d_info = _parse_driver_info(node)
+        managers = redfish_utils.get_system(task.node).managers
+
+        if manager_utils.is_fast_track(task):
+            if _has_vmedia_device(managers, sushy.VIRTUAL_MEDIA_CD,
+                                  inserted=True):
+                LOG.debug('Fast track operation for node %s, not inserting '
+                          'any devices', node.uuid)
+                return
+            else:
+                LOG.warning('Fast track is possible for node %s, but no ISO '
+                            'is currently inserted! Proceeding with '
+                            'normal operation.', node.uuid)
+
         # NOTE(TheJulia): Since we're deploying, cleaning, or rescuing,
         # with virtual media boot, we should generate a token!
-        manager_utils.add_secret_token(node, pregenerated=True)
-        node.save()
-        ramdisk_params['ipa-agent-token'] = \
-            node.driver_internal_info['agent_secret_token']
+        # However, we don't have a way to inject it with a pre-built ISO
+        # if a removable disk is not used.
+        can_config = d_info.pop('can_provide_config', True)
+        if can_config:
+            manager_utils.add_secret_token(node, pregenerated=True)
+            node.save()
+            ramdisk_params['ipa-agent-token'] = \
+                node.driver_internal_info['agent_secret_token']
 
         manager_utils.node_power_action(task, states.POWER_OFF)
 
-        d_info = _parse_driver_info(node)
-
-        config_via_floppy = d_info.get('config_via_floppy')
-
         deploy_nic_mac = deploy_utils.get_single_nic_with_vif_port_id(task)
         if deploy_nic_mac is not None:
             ramdisk_params['BOOTIF'] = deploy_nic_mac
         if CONF.debug and 'ipa-debug' not in ramdisk_params:
             ramdisk_params['ipa-debug'] = '1'
 
-        managers = redfish_utils.get_system(task.node).managers
+        # NOTE(TheJulia): This is a mandatory setting for virtual media
+        # based deployment operations.
+        ramdisk_params['boot_method'] = 'vmedia'
 
-        if config_via_floppy:
-
-            if _has_vmedia_device(managers, sushy.VIRTUAL_MEDIA_FLOPPY):
-                # NOTE (etingof): IPA will read the diskette only if
-                # we tell it to
-                ramdisk_params['boot_method'] = 'vmedia'
+        config_via_removable = d_info.get('config_via_removable')
+        if config_via_removable:
 
+            removable = _has_vmedia_device(
+                managers,
+                # Prefer USB devices since floppies are outdated
+                [sushy.VIRTUAL_MEDIA_USBSTICK, sushy.VIRTUAL_MEDIA_FLOPPY])
+            if removable:
                 floppy_ref = image_utils.prepare_floppy_image(
                     task, params=ramdisk_params)
 
-                _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_FLOPPY)
-                _insert_vmedia(
-                    task, managers, floppy_ref, sushy.VIRTUAL_MEDIA_FLOPPY)
+                _eject_vmedia(task, managers, removable)
+                _insert_vmedia(task, managers, floppy_ref, removable)
 
-                LOG.debug('Inserted virtual floppy with configuration for '
-                          'node %(node)s', {'node': task.node.uuid})
+                LOG.info('Inserted virtual %(type)s device with configuration'
+                         ' for node %(node)s',
+                         {'node': task.node.uuid, 'type': removable})
 
             else:
-                LOG.warning('Config via floppy is requested, but '
-                            'Floppy drive is not available on node '
-                            '%(node)s', {'node': task.node.uuid})
+                LOG.warning('Config via a removable device is requested, but '
+                            'virtual USB and floppy devices are not '
+                            'available on node %(node)s',
+                            {'node': task.node.uuid})
 
         mode = deploy_utils.rescue_or_deploy_mode(node)
 
@@ -517,23 +578,14 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         :param task: A task from TaskManager.
         :returns: None
         """
-        d_info = _parse_driver_info(task.node)
-
-        config_via_floppy = d_info.get('config_via_floppy')
+        if manager_utils.is_fast_track(task):
+            LOG.debug('Fast track operation for node %s, not ejecting '
+                      'any devices', task.node.uuid)
+            return
 
         LOG.debug("Cleaning up deploy boot for "
                   "%(node)s", {'node': task.node.uuid})
-
-        managers = redfish_utils.get_system(task.node).managers
-
-        _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_CD)
-        image_utils.cleanup_iso_image(task)
-
-        if (config_via_floppy
-                and _has_vmedia_device(managers, sushy.VIRTUAL_MEDIA_FLOPPY)):
-            _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_FLOPPY)
-
-            image_utils.cleanup_floppy_image(task)
+        self._eject_all(task)
 
     def prepare_instance(self, task):
         """Prepares the boot of instance over virtual media.
@@ -590,21 +642,12 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         managers = redfish_utils.get_system(task.node).managers
 
         deploy_info = _parse_deploy_info(node)
-        configdrive = node.instance_info.get('configdrive')
         iso_ref = image_utils.prepare_boot_iso(task, deploy_info, **params)
         _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_CD)
         _insert_vmedia(task, managers, iso_ref, sushy.VIRTUAL_MEDIA_CD)
 
-        if configdrive and boot_option == 'ramdisk':
-            _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_USBSTICK)
-            cd_ref = image_utils.prepare_configdrive_image(task, configdrive)
-            try:
-                _insert_vmedia(task, managers, cd_ref,
-                               sushy.VIRTUAL_MEDIA_USBSTICK)
-            except exception.InvalidParameterValue:
-                raise exception.InstanceDeployFailure(
-                    _('Cannot attach configdrive for node %s: no suitable '
-                      'virtual USB slot has been found') % node.uuid)
+        if boot_option == 'ramdisk':
+            self._attach_configdrive(task, managers)
 
         del managers
 
@@ -614,18 +657,38 @@ class RedfishVirtualMediaBoot(base.BootInterface):
                   "%(device)s", {'node': task.node.uuid,
                                  'device': boot_devices.CDROM})
 
+    def _attach_configdrive(self, task, managers):
+        configdrive = manager_utils.get_configdrive_image(task.node)
+        if not configdrive:
+            return
+
+        _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_USBSTICK)
+        cd_ref = image_utils.prepare_configdrive_image(task, configdrive)
+        try:
+            _insert_vmedia(task, managers, cd_ref,
+                           sushy.VIRTUAL_MEDIA_USBSTICK)
+        except exception.InvalidParameterValue:
+            raise exception.InstanceDeployFailure(
+                _('Cannot attach configdrive for node %s: no suitable '
+                  'virtual USB slot has been found') % task.node.uuid)
+
     def _eject_all(self, task):
         managers = redfish_utils.get_system(task.node).managers
 
         _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_CD)
-        d_info = task.node.driver_info
-        config_via_floppy = d_info.get('config_via_floppy')
-        if config_via_floppy:
+        config_via_removable = (
+            task.node.driver_info.get('config_via_removable')
+            or task.node.driver_info.get('config_via_floppy')
+        )
+        if config_via_removable:
+            _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_USBSTICK)
             _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_FLOPPY)
 
+            image_utils.cleanup_floppy_image(task)
+
         boot_option = deploy_utils.get_boot_option(task.node)
         if (boot_option == 'ramdisk'
-                and task.node.instance_info.get('configdrive')):
+                and task.node.instance_info.get('configdrive') is not None):
             _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_USBSTICK)
             image_utils.cleanup_disk_image(task, prefix='configdrive')
 
