@@ -17,7 +17,6 @@ import json
 from unittest import mock
 
 import requests
-import retrying
 
 from ironic.common import exception
 from ironic import conf
@@ -109,9 +108,9 @@ class TestAgentClient(base.TestCase):
                           self.node)
 
     def test__get_command_body(self):
-        expected = json.dumps({'name': 'prepare_image', 'params': {}})
+        expected = json.dumps({'name': 'get_clean_steps', 'params': {}})
         self.assertEqual(expected,
-                         self.client._get_command_body('prepare_image', {}))
+                         self.client._get_command_body('get_clean_steps', {}))
 
     def test__command(self):
         response_data = {'status': 'ok'}
@@ -194,6 +193,123 @@ class TestAgentClient(base.TestCase):
             timeout=60,
             verify=True)
         self.assertEqual(3, self.client.session.post.call_count)
+        self.assertTrue(self.client.session.get.called)
+
+    def test__command_fail_connect_no_command_running(self):
+        error = 'Boom'
+        self.client.session.post.side_effect = requests.ConnectionError(error)
+        self.client.session.get.return_value.json.return_value = {
+            'commands': []
+        }
+        method = 'foo.bar'
+        params = {}
+
+        url = self.client._get_command_url(self.node)
+        self.client._get_command_body(method, params)
+
+        e = self.assertRaises(exception.AgentConnectionFailed,
+                              self.client._command,
+                              self.node, method, params)
+        self.assertEqual('Connection to agent failed: Failed to connect to '
+                         'the agent running on node %(node)s for invoking '
+                         'command %(method)s. Error: %(error)s' %
+                         {'method': method, 'node': self.node.uuid,
+                          'error': error}, str(e))
+        self.client.session.post.assert_called_with(
+            url,
+            data=mock.ANY,
+            params={'wait': 'false'},
+            timeout=60,
+            verify=True)
+        self.assertEqual(3, self.client.session.post.call_count)
+        self.assertTrue(self.client.session.get.called)
+
+    def test__command_fail_connect_wrong_command_running(self):
+        error = 'Boom'
+        self.client.session.post.side_effect = requests.ConnectionError(error)
+        self.client.session.get.return_value.json.return_value = {
+            'commands': [
+                {'command_name': 'meow', 'command_status': 'RUNNING'},
+            ]
+        }
+        method = 'foo.bar'
+        params = {}
+
+        url = self.client._get_command_url(self.node)
+        self.client._get_command_body(method, params)
+
+        e = self.assertRaises(exception.AgentConnectionFailed,
+                              self.client._command,
+                              self.node, method, params)
+        self.assertEqual('Connection to agent failed: Failed to connect to '
+                         'the agent running on node %(node)s for invoking '
+                         'command %(method)s. Error: %(error)s' %
+                         {'method': method, 'node': self.node.uuid,
+                          'error': error}, str(e))
+        self.client.session.post.assert_called_with(
+            url,
+            data=mock.ANY,
+            params={'wait': 'false'},
+            timeout=60,
+            verify=True)
+        self.assertEqual(3, self.client.session.post.call_count)
+        self.assertTrue(self.client.session.get.called)
+
+    def test__command_fail_connect_command_not_running(self):
+        error = 'Boom'
+        self.client.session.post.side_effect = requests.ConnectionError(error)
+        self.client.session.get.return_value.json.return_value = {
+            'commands': [
+                {'command_name': 'bar', 'command_status': 'FINISHED'},
+            ]
+        }
+        method = 'foo.bar'
+        params = {}
+
+        url = self.client._get_command_url(self.node)
+        self.client._get_command_body(method, params)
+
+        e = self.assertRaises(exception.AgentConnectionFailed,
+                              self.client._command,
+                              self.node, method, params)
+        self.assertEqual('Connection to agent failed: Failed to connect to '
+                         'the agent running on node %(node)s for invoking '
+                         'command %(method)s. Error: %(error)s' %
+                         {'method': method, 'node': self.node.uuid,
+                          'error': error}, str(e))
+        self.client.session.post.assert_called_with(
+            url,
+            data=mock.ANY,
+            params={'wait': 'false'},
+            timeout=60,
+            verify=True)
+        self.assertEqual(3, self.client.session.post.call_count)
+        self.assertTrue(self.client.session.get.called)
+
+    def test__command_fail_connect_command_is_running(self):
+        error = 'Boom'
+        self.client.session.post.side_effect = requests.ConnectionError(error)
+        self.client.session.get.return_value.json.return_value = {
+            'commands': [
+                {'command_name': 'bar', 'command_status': 'RUNNING'},
+            ]
+        }
+        method = 'foo.bar'
+        params = {}
+
+        url = self.client._get_command_url(self.node)
+        self.client._get_command_body(method, params)
+
+        result = self.client._command(self.node, method, params)
+        self.assertEqual({'command_name': 'bar', 'command_status': 'RUNNING'},
+                         result)
+        self.client.session.post.assert_called_once_with(
+            url,
+            data=mock.ANY,
+            params={'wait': 'false'},
+            timeout=60,
+            verify=True)
+        self.assertTrue(self.client.session.get.called)
 
     def test__command_error_code(self):
         response_text = {"faultstring": "you dun goofd"}
@@ -237,7 +353,53 @@ class TestAgentClient(base.TestCase):
             timeout=60,
             verify=True)
 
-    def test__command_verify(self):
+    def test__command_error_code_agent_busy(self):
+        # Victoria and previous busy status check.
+        response_text = {"faultstring": "Agent is busy - meowing"}
+        self.client.session.post.return_value = MockResponse(
+            response_text, status_code=http_client.BAD_REQUEST)
+        method = 'standby.run_image'
+        image_info = {'image_id': 'test_image'}
+        params = {'image_info': image_info}
+
+        url = self.client._get_command_url(self.node)
+        body = self.client._get_command_body(method, params)
+
+        self.assertRaises(exception.AgentInProgress,
+                          self.client._command,
+                          self.node, method, params)
+        self.client.session.post.assert_called_once_with(
+            url,
+            data=body,
+            params={'wait': 'false'},
+            timeout=60,
+            verify=True)
+
+    def test__command_error_code_agent_busy_conflict(self):
+        # Post Wallaby logic as the IPA return code changed to
+        # better delineate the state.
+        response_text = {"faultstring": "lolcat says busy meowing"}
+        self.client.session.post.return_value = MockResponse(
+            response_text, status_code=http_client.CONFLICT)
+        method = 'standby.run_image'
+        image_info = {'image_id': 'test_image'}
+        params = {'image_info': image_info}
+
+        url = self.client._get_command_url(self.node)
+        body = self.client._get_command_body(method, params)
+
+        self.assertRaises(exception.AgentInProgress,
+                          self.client._command,
+                          self.node, method, params)
+        self.client.session.post.assert_called_once_with(
+            url,
+            data=body,
+            params={'wait': 'false'},
+            timeout=60,
+            verify=True)
+
+    @mock.patch('os.path.exists', autospec=True, return_value=True)
+    def test__command_verify(self, mock_exists):
         response_data = {'status': 'ok'}
         self.client.session.post.return_value = MockResponse(response_data)
         method = 'standby.run_image'
@@ -258,7 +420,86 @@ class TestAgentClient(base.TestCase):
             timeout=60,
             verify='/path/to/agent.crt')
 
-    @mock.patch('time.sleep', lambda seconds: None)
+    @mock.patch('os.path.exists', autospec=True, return_value=True)
+    def test__command_verify_internal(self, mock_exists):
+        response_data = {'status': 'ok'}
+        self.client.session.post.return_value = MockResponse(response_data)
+        method = 'standby.run_image'
+        image_info = {'image_id': 'test_image'}
+        params = {'image_info': image_info}
+
+        self.node.driver_info['agent_verify_ca'] = True
+        self.node.driver_internal_info['agent_verify_ca'] = '/path/to/crt'
+
+        url = self.client._get_command_url(self.node)
+        body = self.client._get_command_body(method, params)
+
+        response = self.client._command(self.node, method, params)
+        self.assertEqual(response, response_data)
+        self.client.session.post.assert_called_once_with(
+            url,
+            data=body,
+            params={'wait': 'false'},
+            timeout=60,
+            verify='/path/to/crt')
+
+    @mock.patch('os.path.exists', autospec=True, return_value=True)
+    def test__command_verify_config(self, mock_exists):
+        response_data = {'status': 'ok'}
+        self.client.session.post.return_value = MockResponse(response_data)
+        method = 'standby.run_image'
+        image_info = {'image_id': 'test_image'}
+        params = {'image_info': image_info}
+
+        self.config(verify_ca='/path/to/crt', group='agent')
+
+        url = self.client._get_command_url(self.node)
+        body = self.client._get_command_body(method, params)
+
+        response = self.client._command(self.node, method, params)
+        self.assertEqual(response, response_data)
+        self.client.session.post.assert_called_once_with(
+            url,
+            data=body,
+            params={'wait': 'false'},
+            timeout=60,
+            verify='/path/to/crt')
+
+    @mock.patch('os.path.exists', autospec=True, return_value=True)
+    def test__command_verify_disable(self, mock_exists):
+        response_data = {'status': 'ok'}
+        self.client.session.post.return_value = MockResponse(response_data)
+        method = 'standby.run_image'
+        image_info = {'image_id': 'test_image'}
+        params = {'image_info': image_info}
+
+        self.config(verify_ca='False', group='agent')
+
+        url = self.client._get_command_url(self.node)
+        body = self.client._get_command_body(method, params)
+
+        response = self.client._command(self.node, method, params)
+        self.assertEqual(response, response_data)
+        self.client.session.post.assert_called_once_with(
+            url,
+            data=body,
+            params={'wait': 'false'},
+            timeout=60,
+            verify=False)
+
+    @mock.patch('os.path.exists', autospec=True, return_value=False)
+    def test__command_verify_invalid_file(self, mock_exists):
+        response_data = {'status': 'ok'}
+        self.client.session.post.return_value = MockResponse(response_data)
+        method = 'standby.run_image'
+        image_info = {'image_id': 'test_image'}
+        params = {'image_info': image_info}
+
+        self.config(verify_ca='/path/to/crt', group='agent')
+
+        self.assertRaises(exception.InvalidParameterValue,
+                          self.client._command, self.node, method, params)
+
     def test__command_poll(self):
         response_data = {'status': 'ok'}
         final_status = MockCommandStatus('SUCCEEDED', name='run_image')
@@ -322,7 +563,8 @@ class TestAgentClient(base.TestCase):
                           retry_connection=False)
         self.assertEqual(1, self.client.session.get.call_count)
 
-    def test_get_commands_status_verify(self):
+    @mock.patch('os.path.exists', autospec=True, return_value=True)
+    def test_get_commands_status_verify(self, mock_exists):
         self.node.driver_info['agent_verify_ca'] = '/path/to/agent.crt'
 
         with mock.patch.object(self.client.session, 'get',
@@ -339,87 +581,6 @@ class TestAgentClient(base.TestCase):
                 timeout=CONF.agent.command_timeout,
                 verify='/path/to/agent.crt')
 
-    def test_prepare_image(self):
-        self.client._command = mock.MagicMock(spec_set=[])
-        image_info = {'image_id': 'image'}
-        params = {'image_info': image_info}
-
-        self.client.prepare_image(self.node,
-                                  image_info,
-                                  wait=False)
-        self.client._command.assert_called_once_with(
-            node=self.node, method='standby.prepare_image',
-            params=params, poll=False)
-
-    def test_prepare_image_with_configdrive(self):
-        self.client._command = mock.MagicMock(spec_set=[])
-        configdrive_url = 'http://swift/configdrive'
-        self.node.instance_info['configdrive'] = configdrive_url
-        image_info = {'image_id': 'image'}
-        params = {
-            'image_info': image_info,
-            'configdrive': configdrive_url,
-        }
-
-        self.client.prepare_image(self.node,
-                                  image_info,
-                                  wait=False)
-        self.client._command.assert_called_once_with(
-            node=self.node, method='standby.prepare_image',
-            params=params, poll=False)
-
-    def test_prepare_image_with_wait(self):
-        self.client._command = mock.MagicMock(spec_set=[])
-        image_info = {'image_id': 'image'}
-        params = {'image_info': image_info}
-
-        self.client.prepare_image(self.node,
-                                  image_info,
-                                  wait=True)
-        self.client._command.assert_called_once_with(
-            node=self.node, method='standby.prepare_image',
-            params=params, poll=True)
-
-    def test_start_iscsi_target(self):
-        self.client._command = mock.MagicMock(spec_set=[])
-        iqn = 'fake-iqn'
-        port = agent_client.DEFAULT_IPA_PORTAL_PORT
-        wipe_disk_metadata = False
-        params = {'iqn': iqn, 'portal_port': port,
-                  'wipe_disk_metadata': wipe_disk_metadata}
-
-        self.client.start_iscsi_target(self.node, iqn)
-        self.client._command.assert_called_once_with(
-            node=self.node, method='iscsi.start_iscsi_target',
-            params=params, wait=True)
-
-    def test_start_iscsi_target_custom_port(self):
-        self.client._command = mock.MagicMock(spec_set=[])
-        iqn = 'fake-iqn'
-        port = 3261
-        wipe_disk_metadata = False
-        params = {'iqn': iqn, 'portal_port': port,
-                  'wipe_disk_metadata': wipe_disk_metadata}
-
-        self.client.start_iscsi_target(self.node, iqn, portal_port=port)
-        self.client._command.assert_called_once_with(
-            node=self.node, method='iscsi.start_iscsi_target',
-            params=params, wait=True)
-
-    def test_start_iscsi_target_wipe_disk_metadata(self):
-        self.client._command = mock.MagicMock(spec_set=[])
-        iqn = 'fake-iqn'
-        port = agent_client.DEFAULT_IPA_PORTAL_PORT
-        wipe_disk_metadata = True
-        params = {'iqn': iqn, 'portal_port': port,
-                  'wipe_disk_metadata': wipe_disk_metadata}
-
-        self.client.start_iscsi_target(self.node, iqn,
-                                       wipe_disk_metadata=wipe_disk_metadata)
-        self.client._command.assert_called_once_with(
-            node=self.node, method='iscsi.start_iscsi_target',
-            params=params, wait=True)
-
     def _test_install_bootloader(self, root_uuid, efi_system_part_uuid=None,
                                  prep_boot_part_uuid=None):
         self.client._command = mock.MagicMock(spec_set=[])
@@ -569,9 +730,7 @@ class TestAgentClientAttempts(base.TestCase):
         self.client.session = mock.MagicMock(autospec=requests.Session)
         self.node = MockNode()
 
-    @mock.patch.object(retrying.time, 'sleep', autospec=True)
-    def test__command_fail_all_attempts(self, mock_sleep):
-        mock_sleep.return_value = None
+    def test__command_fail_all_attempts(self):
         error = 'Connection Timeout'
         method = 'standby.run_image'
         image_info = {'image_id': 'test_image'}
@@ -593,9 +752,7 @@ class TestAgentClientAttempts(base.TestCase):
                           'error': error}, str(e))
         self.assertEqual(3, self.client.session.post.call_count)
 
-    @mock.patch.object(retrying.time, 'sleep', autospec=True)
-    def test__command_succeed_after_two_timeouts(self, mock_sleep):
-        mock_sleep.return_value = None
+    def test__command_succeed_after_two_timeouts(self):
         error = 'Connection Timeout'
         response_data = {'status': 'ok'}
         method = 'standby.run_image'
@@ -615,37 +772,7 @@ class TestAgentClientAttempts(base.TestCase):
             timeout=60,
             verify=True)
 
-    @mock.patch.object(retrying.time, 'sleep', autospec=True)
-    def test__command_succeed_after_agent_token(self, mock_sleep):
-        self.config(require_agent_token=False)
-        mock_sleep.return_value = None
-        error = 'Unknown Argument: "agent_token"'
-        response_data = {'status': 'ok'}
-        method = 'standby.run_image'
-        image_info = {'image_id': 'test_image'}
-        params = {'image_info': image_info}
-        i_info = self.node.driver_internal_info
-        i_info['agent_secret_token'] = 'meowmeowmeow'
-        self.client.session.post.side_effect = [
-            MockFault(error),
-            MockResponse(response_data),
-        ]
-
-        response = self.client._command(self.node, method, params)
-        self.assertEqual(2, self.client.session.post.call_count)
-        self.assertEqual(response, response_data)
-        self.client.session.post.assert_called_with(
-            self.client._get_command_url(self.node),
-            data=self.client._get_command_body(method, params),
-            params={'wait': 'false'},
-            timeout=60,
-            verify=True)
-        self.assertNotIn('agent_secret_token', self.node.driver_internal_info)
-
-    @mock.patch.object(retrying.time, 'sleep', autospec=True)
-    def test__command_fail_agent_token_required(self, mock_sleep):
-        self.config(require_agent_token=True)
-        mock_sleep.return_value = None
+    def test__command_fail_agent_token_required(self):
         error = 'Unknown Argument: "agent_token"'
         method = 'standby.run_image'
         image_info = {'image_id': 'test_image'}
@@ -670,9 +797,7 @@ class TestAgentClientAttempts(base.TestCase):
             'meowmeowmeow',
             self.node.driver_internal_info.get('agent_secret_token'))
 
-    @mock.patch.object(retrying.time, 'sleep', autospec=True)
-    def test__command_succeed_after_one_timeout(self, mock_sleep):
-        mock_sleep.return_value = None
+    def test__command_succeed_after_one_timeout(self):
         error = 'Connection Timeout'
         response_data = {'status': 'ok'}
         method = 'standby.run_image'
