@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
 import os
 import tempfile
 from unittest import mock
@@ -30,6 +31,7 @@ from ironic.tests.unit.objects import utils as obj_utils
 sushy = importutils.try_import('sushy')
 
 INFO_DICT = db_utils.get_test_redfish_info()
+INFO_DICT_ILO = db_utils.get_test_ilo_info()
 
 
 class RedfishImageHandlerTestCase(db_base.DbTestCase):
@@ -100,19 +102,15 @@ class RedfishImageHandlerTestCase(db_base.DbTestCase):
             mock_swift_api.delete_object.assert_called_once_with(
                 'ironic_redfish_container', object_name)
 
-    @mock.patch.object(image_utils.ImageHandler, '_is_swift_enabled',
-                       autospec=True)
     @mock.patch.object(os, 'chmod', autospec=True)
     @mock.patch.object(image_utils, 'shutil', autospec=True)
     @mock.patch.object(os, 'link', autospec=True)
     @mock.patch.object(os, 'mkdir', autospec=True)
     def test_publish_image_local_link(
-            self, mock_mkdir, mock_link, mock_shutil, mock_chmod,
-            mock__is_swift):
-        img_handler_obj = image_utils.ImageHandler(self.node.driver)
-        mock__is_swift.return_value = False
+            self, mock_mkdir, mock_link, mock_shutil, mock_chmod):
         self.config(use_swift=False, group='redfish')
         self.config(http_url='http://localhost', group='deploy')
+        img_handler_obj = image_utils.ImageHandler(self.node.driver)
 
         url = img_handler_obj.publish_image('file.iso', 'boot.iso')
 
@@ -124,16 +122,34 @@ class RedfishImageHandlerTestCase(db_base.DbTestCase):
             'file.iso', '/httpboot/redfish/boot.iso')
         mock_chmod.assert_called_once_with('file.iso', 0o644)
 
-    @mock.patch.object(image_utils.ImageHandler, '_is_swift_enabled',
-                       autospec=True)
+    @mock.patch.object(os, 'chmod', autospec=True)
+    @mock.patch.object(image_utils, 'shutil', autospec=True)
+    @mock.patch.object(os, 'link', autospec=True)
+    @mock.patch.object(os, 'mkdir', autospec=True)
+    def test_publish_image_external_ip(
+            self, mock_mkdir, mock_link, mock_shutil, mock_chmod):
+        self.config(use_swift=False, group='redfish')
+        self.config(http_url='http://localhost',
+                    external_http_url='http://non-local.host',
+                    group='deploy')
+        img_handler_obj = image_utils.ImageHandler(self.node.driver)
+
+        url = img_handler_obj.publish_image('file.iso', 'boot.iso')
+
+        self.assertEqual(
+            'http://non-local.host/redfish/boot.iso?filename=file.iso', url)
+
+        mock_mkdir.assert_called_once_with('/httpboot/redfish', 0o755)
+        mock_link.assert_called_once_with(
+            'file.iso', '/httpboot/redfish/boot.iso')
+        mock_chmod.assert_called_once_with('file.iso', 0o644)
+
     @mock.patch.object(os, 'chmod', autospec=True)
     @mock.patch.object(image_utils, 'shutil', autospec=True)
     @mock.patch.object(os, 'link', autospec=True)
     @mock.patch.object(os, 'mkdir', autospec=True)
     def test_publish_image_local_copy(self, mock_mkdir, mock_link,
-                                      mock_shutil, mock_chmod,
-                                      mock__is_swift):
-        mock__is_swift.return_value = False
+                                      mock_shutil, mock_chmod):
         self.config(use_swift=False, group='redfish')
         self.config(http_url='http://localhost', group='deploy')
         img_handler_obj = image_utils.ImageHandler(self.node.driver)
@@ -152,12 +168,9 @@ class RedfishImageHandlerTestCase(db_base.DbTestCase):
         mock_chmod.assert_called_once_with('/httpboot/redfish/boot.iso',
                                            0o644)
 
-    @mock.patch.object(image_utils.ImageHandler, '_is_swift_enabled',
-                       autospec=True)
     @mock.patch.object(image_utils, 'ironic_utils', autospec=True)
-    def test_unpublish_image_local(self, mock_ironic_utils, mock__is_swift):
+    def test_unpublish_image_local(self, mock_ironic_utils):
         self.config(use_swift=False, group='redfish')
-        mock__is_swift.return_value = False
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             img_handler_obj = image_utils.ImageHandler(self.node.driver)
@@ -171,6 +184,50 @@ class RedfishImageHandlerTestCase(db_base.DbTestCase):
                 expected_file)
 
 
+class IloImageHandlerTestCase(db_base.DbTestCase):
+
+    def setUp(self):
+        super(IloImageHandlerTestCase, self).setUp()
+        self.config(enabled_hardware_types=['ilo'],
+                    enabled_power_interfaces=['ilo'],
+                    enabled_boot_interfaces=['ilo-virtual-media'],
+                    enabled_management_interfaces=['ilo'],
+                    enabled_inspect_interfaces=['ilo'],
+                    enabled_bios_interfaces=['ilo'])
+        self.node = obj_utils.create_test_node(
+            self.context, driver='ilo', driver_info=INFO_DICT_ILO)
+
+    def test_ilo_kernel_param_config(self):
+        self.config(kernel_append_params="console=ttyS1", group='ilo')
+        img_handler_obj = image_utils.ImageHandler(self.node.driver)
+        actual_k_param = img_handler_obj.kernel_params
+        expected_k_param = "console=ttyS1"
+
+        self.assertEqual(expected_k_param, actual_k_param)
+
+
+class Ilo5ImageHandlerTestCase(db_base.DbTestCase):
+
+    def setUp(self):
+        super(Ilo5ImageHandlerTestCase, self).setUp()
+        self.config(enabled_hardware_types=['ilo5'],
+                    enabled_power_interfaces=['ilo'],
+                    enabled_boot_interfaces=['ilo-virtual-media'],
+                    enabled_management_interfaces=['ilo5'],
+                    enabled_inspect_interfaces=['ilo'],
+                    enabled_bios_interfaces=['ilo'])
+        self.node = obj_utils.create_test_node(
+            self.context, driver='ilo5', driver_info=INFO_DICT_ILO)
+
+    def test_ilo5_kernel_param_config(self):
+        self.config(kernel_append_params="console=ttyS1", group='ilo')
+        img_handler_obj = image_utils.ImageHandler(self.node.driver)
+        actual_k_param = img_handler_obj.kernel_params
+        expected_k_param = "console=ttyS1"
+
+        self.assertEqual(expected_k_param, actual_k_param)
+
+
 class RedfishImageUtilsTestCase(db_base.DbTestCase):
 
     def setUp(self):
@@ -214,7 +271,31 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
                                                        mock.ANY, object_name)
 
             mock_create_vfat_image.assert_called_once_with(
-                mock.ANY, parameters=mock.ANY)
+                mock.ANY, parameters=None)
+
+            self.assertEqual(expected_url, url)
+
+    @mock.patch.object(image_utils.ImageHandler, 'publish_image',
+                       autospec=True)
+    @mock.patch.object(images, 'create_vfat_image', autospec=True)
+    def test_prepare_floppy_image_with_external_ip(
+            self, mock_create_vfat_image, mock_publish_image):
+        self.config(external_callback_url='http://callback/', group='deploy')
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            expected_url = 'https://a.b/c.f?e=f'
+
+            mock_publish_image.return_value = expected_url
+
+            url = image_utils.prepare_floppy_image(task)
+
+            object_name = 'image-%s' % task.node.uuid
+
+            mock_publish_image.assert_called_once_with(mock.ANY,
+                                                       mock.ANY, object_name)
+
+            mock_create_vfat_image.assert_called_once_with(
+                mock.ANY, parameters={"ipa-api-url": "http://callback"})
 
             self.assertEqual(expected_url, url)
 
@@ -473,6 +554,82 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
                 root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
                 base_iso='/path/to/baseiso', inject_files=None)
 
+    @mock.patch.object(image_utils.ImageHandler, 'publish_image',
+                       autospec=True)
+    @mock.patch.object(images, 'create_boot_iso', autospec=True)
+    def test__prepare_iso_image_kernel_params_driver_info(
+            self, mock_create_boot_iso, mock_publish_image):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            kernel_params = 'network-config=base64-cloudinit-blob'
+
+            task.node.driver_info['kernel_append_params'] = kernel_params
+
+            image_utils._prepare_iso_image(
+                task, 'http://kernel/img', 'http://ramdisk/img',
+                bootloader_href=None, root_uuid=task.node.uuid,
+                base_iso='/path/to/baseiso')
+
+            mock_create_boot_iso.assert_called_once_with(
+                mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
+                boot_mode='bios', esp_image_href=None,
+                kernel_params=kernel_params,
+                root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
+                base_iso='/path/to/baseiso', inject_files=None)
+
+    @mock.patch.object(image_utils.ImageHandler, 'publish_image',
+                       autospec=True)
+    @mock.patch.object(images, 'create_boot_iso', autospec=True)
+    def test__prepare_iso_image_extra_params(
+            self, mock_create_boot_iso, mock_publish_image):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            kernel_params = 'network-config=base64-cloudinit-blob'
+            extra_params = collections.OrderedDict([('foo', 'bar'),
+                                                    ('banana', None)])
+            self.config(kernel_append_params=kernel_params, group='redfish')
+
+            image_utils._prepare_iso_image(
+                task, 'http://kernel/img', 'http://ramdisk/img',
+                root_uuid=task.node.uuid, params=extra_params)
+
+            mock_create_boot_iso.assert_called_once_with(
+                mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
+                boot_mode='bios', esp_image_href=None,
+                kernel_params=kernel_params + ' foo=bar banana',
+                root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
+                base_iso=None, inject_files=None)
+
+    def test__prepare_iso_image_bootable_iso(self):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            base_image_url = 'http://bearmetal.net/boot.iso'
+            self.config(ramdisk_image_download_source='http', group='deploy')
+            url = image_utils._prepare_iso_image(
+                task, None, None, bootloader_href=None, root_uuid=None,
+                base_iso=base_image_url)
+            self.assertEqual(url, base_image_url)
+
+    @mock.patch.object(image_utils.ImageHandler, 'publish_image',
+                       autospec=True)
+    @mock.patch.object(images, 'create_boot_iso', autospec=True)
+    def test__prepare_iso_image_bootable_iso_file(self, mock_create_boot_iso,
+                                                  mock_publish_image):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            base_image_url = '/path/to/baseiso'
+            self.config(ramdisk_image_download_source='http', group='deploy')
+            image_utils._prepare_iso_image(
+                task, 'http://kernel/img', 'http://ramdisk/img',
+                bootloader_href=None, root_uuid=task.node.uuid,
+                base_iso=base_image_url)
+            mock_create_boot_iso.assert_called_once_with(
+                mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
+                esp_image_href=None,
+                root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
+                kernel_params='nofb nomodeset vga=normal', boot_mode='bios',
+                base_iso='/path/to/baseiso', inject_files=None)
+
     def test__find_param(self):
         param_dict = {
             'ilo_deploy_kernel': 'kernel',
@@ -495,10 +652,8 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
         actual = image_utils._find_param(param_str, param_dict)
         self.assertEqual(actual, expected)
 
-    @mock.patch.object(image_utils, '_find_param', autospec=True)
     @mock.patch.object(image_utils, '_prepare_iso_image', autospec=True)
-    def test_prepare_deploy_iso(self, mock__prepare_iso_image,
-                                find_mock):
+    def test_prepare_deploy_iso(self, mock__prepare_iso_image):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
 
@@ -509,29 +664,53 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
             }
             task.node.driver_info.update(d_info)
 
-            find_call_list = [
-                mock.call('deploy_kernel', d_info),
-                mock.call('deploy_ramdisk', d_info),
-                mock.call('bootloader', d_info)
-            ]
-            find_mock.side_effect = [
-                'kernel', 'ramdisk', 'bootloader'
-            ]
-
             task.node.instance_info.update(deploy_boot_mode='uefi')
 
             image_utils.prepare_deploy_iso(task, {}, 'deploy', d_info)
 
             mock__prepare_iso_image.assert_called_once_with(
                 task, 'kernel', 'ramdisk', 'bootloader', params={},
-                inject_files={})
+                inject_files={}, base_iso=None)
 
-            find_mock.assert_has_calls(find_call_list)
-
-    @mock.patch.object(image_utils, '_find_param', autospec=True)
     @mock.patch.object(image_utils, '_prepare_iso_image', autospec=True)
-    def test_prepare_deploy_iso_network_data(
-            self, mock__prepare_iso_image, find_mock):
+    def test_prepare_deploy_iso_existing_iso(self, mock__prepare_iso_image):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+
+            d_info = {
+                'deploy_iso': 'iso',
+            }
+            task.node.driver_info.update(d_info)
+
+            task.node.instance_info.update(deploy_boot_mode='uefi')
+
+            image_utils.prepare_deploy_iso(task, {}, 'deploy', d_info)
+
+            mock__prepare_iso_image.assert_called_once_with(
+                task, None, None, None, params={},
+                inject_files={}, base_iso='iso')
+
+    @mock.patch.object(image_utils, '_prepare_iso_image', autospec=True)
+    def test_prepare_deploy_iso_existing_iso_vendor_prefix(
+            self, mock__prepare_iso_image):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+
+            d_info = {
+                'redfish_deploy_iso': 'iso',
+            }
+            task.node.driver_info.update(d_info)
+
+            task.node.instance_info.update(deploy_boot_mode='uefi')
+
+            image_utils.prepare_deploy_iso(task, {}, 'deploy', d_info)
+
+            mock__prepare_iso_image.assert_called_once_with(
+                task, None, None, None, params={},
+                inject_files={}, base_iso='iso')
+
+    @mock.patch.object(image_utils, '_prepare_iso_image', autospec=True)
+    def test_prepare_deploy_iso_network_data(self, mock__prepare_iso_image):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
 
@@ -543,14 +722,6 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
 
             task.node.instance_info.update()
 
-            find_call_list = [
-                mock.call('deploy_kernel', d_info),
-                mock.call('deploy_ramdisk', d_info),
-                mock.call('bootloader', d_info)
-            ]
-            find_mock.side_effect = [
-                'kernel', 'ramdisk', None
-            ]
             network_data = {'a': ['b']}
 
             mock_get_node_nw_data = mock.MagicMock(return_value=network_data)
@@ -568,14 +739,10 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
 
             mock__prepare_iso_image.assert_called_once_with(
                 task, 'kernel', 'ramdisk', bootloader_href=None,
-                params={}, inject_files=expected_files)
+                params={}, inject_files=expected_files, base_iso=None)
 
-            find_mock.assert_has_calls(find_call_list)
-
-    @mock.patch.object(image_utils, '_find_param', autospec=True)
     @mock.patch.object(image_utils, '_prepare_iso_image', autospec=True)
-    def test_prepare_deploy_iso_tls(self, mock__prepare_iso_image,
-                                    find_mock):
+    def test_prepare_deploy_iso_tls(self, mock__prepare_iso_image):
         with tempfile.NamedTemporaryFile(delete=False) as tf:
             temp_name = tf.name
             self.addCleanup(lambda: os.unlink(temp_name))
@@ -592,15 +759,6 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
             }
             task.node.driver_info.update(d_info)
 
-            find_call_list = [
-                mock.call('deploy_kernel', d_info),
-                mock.call('deploy_ramdisk', d_info),
-                mock.call('bootloader', d_info)
-            ]
-            find_mock.side_effect = [
-                'kernel', 'ramdisk', 'bootloader'
-            ]
-
             task.node.instance_info.update(deploy_boot_mode='uefi')
 
             image_utils.prepare_deploy_iso(task, {}, 'deploy', d_info)
@@ -614,9 +772,29 @@ cafile = /etc/ironic-python-agent/ironic.crt
 
             mock__prepare_iso_image.assert_called_once_with(
                 task, 'kernel', 'ramdisk', 'bootloader', params={},
-                inject_files=expected_files)
+                inject_files=expected_files, base_iso=None)
 
-            find_mock.assert_has_calls(find_call_list)
+    @mock.patch.object(image_utils, '_prepare_iso_image', autospec=True)
+    def test_prepare_deploy_iso_external_ip(self, mock__prepare_iso_image):
+        self.config(external_callback_url='http://callback/', group='deploy')
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+
+            d_info = {
+                'ilo_deploy_kernel': 'kernel',
+                'ilo_deploy_ramdisk': 'ramdisk',
+                'ilo_bootloader': 'bootloader'
+            }
+            task.node.driver_info.update(d_info)
+
+            task.node.instance_info.update(deploy_boot_mode='uefi')
+
+            image_utils.prepare_deploy_iso(task, {}, 'deploy', d_info)
+
+            mock__prepare_iso_image.assert_called_once_with(
+                task, 'kernel', 'ramdisk', 'bootloader',
+                params={'ipa-api-url': 'http://callback'},
+                inject_files={}, base_iso=None)
 
     @mock.patch.object(image_utils, '_find_param', autospec=True)
     @mock.patch.object(image_utils, '_prepare_iso_image', autospec=True)
