@@ -20,7 +20,6 @@ import tempfile
 from unittest import mock
 
 from oslo_config import cfg
-from oslo_serialization import jsonutils as json
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
 
@@ -42,6 +41,7 @@ from ironic.drivers.modules import ipxe
 from ironic.drivers.modules import pxe
 from ironic.drivers.modules import pxe_base
 from ironic.drivers.modules.storage import noop as noop_storage
+from ironic import objects
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.db import utils as db_utils
 from ironic.tests.unit.objects import utils as obj_utils
@@ -70,11 +70,15 @@ class PXEBootTestCase(db_base.DbTestCase):
         self.config_temp_dir('tftp_root', group='pxe')
         self.config_temp_dir('images_path', group='pxe')
         self.config_temp_dir('http_root', group='deploy')
+        self.config(default_ks_template='/etc/ironic/ks.cfg.template',
+                    group='anaconda')
         instance_info = INST_INFO_DICT
         instance_info['deploy_key'] = 'fake-56789'
+        instance_info['image_url'] = 'http://fakeserver/os.tar.gz'
 
         self.config(enabled_boot_interfaces=[self.boot_interface,
                                              'ipxe', 'fake'])
+        self.config(enabled_deploy_interfaces=['fake', 'direct', 'anaconda'])
         self.node = obj_utils.create_test_node(
             self.context,
             driver=self.driver,
@@ -135,13 +139,18 @@ class PXEBootTestCase(db_base.DbTestCase):
             self.assertRaises(exception.MissingParameterValue,
                               task.driver.boot.validate, task)
 
-    def test_validate_fail_missing_image_source(self):
-        info = dict(INST_INFO_DICT)
-        del info['image_source']
-        self.node.instance_info = json.dumps(info)
+    def test_validate_no_image_source_for_local_boot(self):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
-            task.node['instance_info'] = json.dumps(info)
+            del task.node['instance_info']['image_source']
+            task.driver.boot.validate(task)
+
+    def test_validate_fail_missing_image_source(self):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            task.node['instance_info']['capabilities'] = {
+                'boot_option': 'netboot'}
+            del task.node['instance_info']['image_source']
             self.assertRaises(exception.MissingParameterValue,
                               task.driver.boot.validate, task)
 
@@ -195,6 +204,8 @@ class PXEBootTestCase(db_base.DbTestCase):
         mock_glance.side_effect = exception.ImageNotFound('not found')
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
+            task.node.instance_info['capabilities'] = {
+                'boot_option': 'netboot'}
             self.assertRaises(exception.InvalidParameterValue,
                               task.driver.boot.validate, task)
 
@@ -207,6 +218,8 @@ class PXEBootTestCase(db_base.DbTestCase):
         for exc in exceptions:
             with task_manager.acquire(self.context, self.node.uuid,
                                       shared=True) as task:
+                task.node.instance_info['capabilities'] = {
+                    'boot_option': 'netboot'}
                 self.assertRaises(exception.InvalidParameterValue,
                                   task.driver.boot.validate, task)
 
@@ -223,6 +236,27 @@ class PXEBootTestCase(db_base.DbTestCase):
             self.assertRaises(exception.UnsupportedDriverExtension,
                               task.driver.boot.validate_inspection, task)
 
+    @mock.patch.object(deploy_utils, 'validate_image_properties',
+                       autospec=True)
+    def test_validate_kickstart_has_squashfs_id(self, mock_validate_img):
+        node = self.node
+        node.deploy_interface = 'anaconda'
+        node.save()
+        self.config(http_url='http://fake_url', group='deploy')
+        with task_manager.acquire(self.context, node.uuid) as task:
+            task.driver.boot.validate(task)
+            mock_validate_img.assert_called_once_with(
+                mock.ANY, mock.ANY, ['kernel_id', 'ramdisk_id', 'squashfs_id']
+            )
+
+    def test_validate_kickstart_fail_http_url_not_set(self):
+        node = self.node
+        node.deploy_interface = 'anaconda'
+        node.save()
+        with task_manager.acquire(self.context, node.uuid) as task:
+            self.assertRaises(exception.MissingParameterValue,
+                              task.driver.boot.validate, task)
+
     @mock.patch.object(manager_utils, 'node_get_boot_mode', autospec=True)
     @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
     @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
@@ -502,7 +536,8 @@ class PXEBootTestCase(db_base.DbTestCase):
             provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
             switch_pxe_config_mock.assert_called_once_with(
                 pxe_config_path, "30212642-09d3-467f-8e09-21685826ab50",
-                'bios', False, False, False, False, ipxe_enabled=False)
+                'bios', False, False, False, False, ipxe_enabled=False,
+                anaconda_boot=False)
             set_boot_device_mock.assert_called_once_with(task,
                                                          boot_devices.PXE,
                                                          persistent=True)
@@ -550,7 +585,8 @@ class PXEBootTestCase(db_base.DbTestCase):
                 ipxe_enabled=False)
             switch_pxe_config_mock.assert_called_once_with(
                 pxe_config_path, "30212642-09d3-467f-8e09-21685826ab50",
-                'bios', False, False, False, False, ipxe_enabled=False)
+                'bios', False, False, False, False, ipxe_enabled=False,
+                anaconda_boot=False)
             self.assertFalse(set_boot_device_mock.called)
 
     @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
@@ -702,7 +738,7 @@ class PXEBootTestCase(db_base.DbTestCase):
             switch_pxe_config_mock.assert_called_once_with(
                 pxe_config_path, None,
                 'bios', False, ipxe_enabled=False, iscsi_boot=False,
-                ramdisk_boot=True)
+                ramdisk_boot=True, anaconda_boot=False)
             set_boot_device_mock.assert_called_once_with(task,
                                                          boot_devices.PXE,
                                                          persistent=True)
@@ -715,6 +751,68 @@ class PXEBootTestCase(db_base.DbTestCase):
     def test_prepare_instance_ramdisk_pxe_conf_exists(self):
         self._test_prepare_instance_ramdisk(config_file_exits=False)
 
+    @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
+    @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
+    @mock.patch.object(pxe_utils, 'create_pxe_config', autospec=True)
+    @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
+    @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
+    @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
+    @mock.patch('ironic.drivers.modules.deploy_utils.get_boot_option',
+                return_value='kickstart', autospec=True)
+    @mock.patch('ironic.drivers.modules.deploy_utils.get_ironic_api_url',
+                return_value='http://fakeserver/api', autospec=True)
+    @mock.patch('ironic.common.utils.render_template', autospec=True)
+    @mock.patch('ironic.common.utils.write_to_file', autospec=True)
+    @mock.patch('ironic.common.utils.execute', autospec=True)
+    def test_prepare_instance_kickstart(
+            self, exec_mock, write_file_mock, render_mock, api_url_mock,
+            boot_opt_mock, get_image_info_mock, cache_mock, dhcp_factory_mock,
+            create_pxe_config_mock, switch_pxe_config_mock,
+            set_boot_device_mock):
+        image_info = {'kernel': ['ins_kernel_id', '/path/to/kernel'],
+                      'ramdisk': ['ins_ramdisk_id', '/path/to/ramdisk'],
+                      'stage2': ['ins_stage2_id', '/path/to/stage2'],
+                      'ks_cfg': ['', '/path/to/ks.cfg'],
+                      'ks_template': ['template_id', '/path/to/ks_template']}
+        get_image_info_mock.return_value = image_info
+        provider_mock = mock.MagicMock()
+        dhcp_factory_mock.return_value = provider_mock
+        self.node.provision_state = states.DEPLOYING
+        self.config(http_url='http://fake_url', group='deploy')
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            dhcp_opts = pxe_utils.dhcp_options_for_instance(
+                task, ipxe_enabled=False)
+            dhcp_opts += pxe_utils.dhcp_options_for_instance(
+                task, ipxe_enabled=False, ip_version=6)
+            pxe_config_path = pxe_utils.get_pxe_config_file_path(
+                task.node.uuid)
+
+            task.driver.boot.prepare_instance(task)
+
+            get_image_info_mock.assert_called_once_with(task,
+                                                        ipxe_enabled=False)
+            cache_mock.assert_called_once_with(
+                task, image_info, False)
+            if os.path.isfile('/usr/bin/ksvalidator'):
+                exec_mock.assert_called_once_with(
+                    'ksvalidator', mock.ANY, check_on_exit=[0], attempts=1
+                )
+            provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
+            render_mock.assert_called()
+            write_file_mock.assert_called_with(
+                '/path/to/ks.cfg', render_mock.return_value
+            )
+            create_pxe_config_mock.assert_called_once_with(
+                task, mock.ANY, CONF.pxe.pxe_config_template,
+                ipxe_enabled=False)
+            switch_pxe_config_mock.assert_called_once_with(
+                pxe_config_path, None,
+                'bios', False, ipxe_enabled=False, iscsi_boot=False,
+                ramdisk_boot=False, anaconda_boot=True)
+            set_boot_device_mock.assert_called_once_with(task,
+                                                         boot_devices.PXE,
+                                                         persistent=True)
+
     @mock.patch.object(boot_mode_utils, 'deconfigure_secure_boot_if_needed',
                        autospec=True)
     @mock.patch.object(pxe_utils, 'clean_up_pxe_env', autospec=True)
@@ -801,7 +899,7 @@ class PXERamdiskDeployTestCase(db_base.DbTestCase):
             switch_pxe_config_mock.assert_called_once_with(
                 pxe_config_path, None,
                 'bios', False, ipxe_enabled=False, iscsi_boot=False,
-                ramdisk_boot=True)
+                ramdisk_boot=True, anaconda_boot=False)
             set_boot_device_mock.assert_called_once_with(task,
                                                          boot_devices.PXE,
                                                          persistent=True)
@@ -843,8 +941,6 @@ class PXERamdiskDeployTestCase(db_base.DbTestCase):
         with task_manager.acquire(self.context, node.uuid) as task:
             task.driver.deploy.prepare(task)
             self.assertFalse(mock_prepare_instance.called)
-            self.assertEqual({'boot_option': 'ramdisk'},
-                             task.node.instance_info['capabilities'])
 
     @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
     def test_prepare_active(self, mock_prepare_instance):
@@ -864,29 +960,12 @@ class PXERamdiskDeployTestCase(db_base.DbTestCase):
             task.driver.deploy.prepare(task)
             mock_prepare_instance.assert_called_once_with(mock.ANY, task)
 
-    @mock.patch.object(pxe.LOG, 'warning', autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
-    def test_prepare_fixes_and_logs_boot_option_warning(
-            self, mock_prepare_instance, mock_warning):
-        node = self.node
-        node.properties['capabilities'] = 'boot_option:ramdisk'
-        node.provision_state = states.DEPLOYING
-        node.instance_info = {}
-        node.save()
-        with task_manager.acquire(self.context, node.uuid) as task:
-            task.driver.deploy.prepare(task)
-            self.assertFalse(mock_prepare_instance.called)
-            self.assertEqual({'boot_option': 'ramdisk'},
-                             task.node.instance_info['capabilities'])
-            self.assertTrue(mock_warning.called)
-
     @mock.patch.object(deploy_utils, 'validate_image_properties',
                        autospec=True)
     def test_validate(self, mock_validate_img):
-        node = self.node
-        node.properties['capabilities'] = 'boot_option:netboot'
-        node.save()
-        with task_manager.acquire(self.context, node.uuid) as task:
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            task.node.instance_info['capabilities'] = {
+                'boot_option': 'netboot'}
             task.driver.deploy.validate(task)
         self.assertTrue(mock_validate_img.called)
 
@@ -955,6 +1034,161 @@ class PXERamdiskDeployTestCase(db_base.DbTestCase):
             self.assertTrue(mock_warning.called)
 
 
+class PXEAnacondaDeployTestCase(db_base.DbTestCase):
+
+    def setUp(self):
+        super(PXEAnacondaDeployTestCase, self).setUp()
+        self.temp_dir = tempfile.mkdtemp()
+        self.config(tftp_root=self.temp_dir, group='pxe')
+        self.config_temp_dir('http_root', group='deploy')
+        self.config(http_url='http://fakeurl', group='deploy')
+        self.temp_dir = tempfile.mkdtemp()
+        self.config(images_path=self.temp_dir, group='pxe')
+        self.config(enabled_deploy_interfaces=['anaconda'])
+        self.config(enabled_boot_interfaces=['pxe'])
+        for iface in drivers_base.ALL_INTERFACES:
+            impl = 'fake'
+            if iface == 'network':
+                impl = 'noop'
+            if iface == 'deploy':
+                impl = 'anaconda'
+            if iface == 'boot':
+                impl = 'pxe'
+            config_kwarg = {'enabled_%s_interfaces' % iface: [impl],
+                            'default_%s_interface' % iface: impl}
+            self.config(**config_kwarg)
+        self.config(enabled_hardware_types=['fake-hardware'])
+        instance_info = INST_INFO_DICT
+        self.node = obj_utils.create_test_node(
+            self.context,
+            driver='fake-hardware',
+            instance_info=instance_info,
+            driver_info=DRV_INFO_DICT,
+            driver_internal_info=DRV_INTERNAL_INFO_DICT)
+        self.port = obj_utils.create_test_port(self.context,
+                                               node_id=self.node.id)
+        self.deploy = pxe.PXEAnacondaDeploy()
+
+    @mock.patch.object(pxe_utils, 'prepare_instance_kickstart_config',
+                       autospec=True)
+    @mock.patch.object(pxe_utils, 'validate_kickstart_file', autospec=True)
+    @mock.patch.object(pxe_utils, 'validate_kickstart_template', autospec=True)
+    @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
+    @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
+    @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
+    @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
+    def test_deploy(self, mock_image_info, mock_cache,
+                    mock_dhcp_factory, mock_switch_config, mock_ks_tmpl,
+                    mock_ks_file, mock_prepare_ks_config):
+        image_info = {'kernel': ('', '/path/to/kernel'),
+                      'ramdisk': ('', '/path/to/ramdisk'),
+                      'stage2': ('', '/path/to/stage2'),
+                      'ks_template': ('', '/path/to/ks_template'),
+                      'ks_cfg': ('', '/path/to/ks_cfg')}
+        mock_image_info.return_value = image_info
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            self.assertIsNone(task.driver.deploy.deploy(task))
+            mock_image_info.assert_called_once_with(task, ipxe_enabled=False)
+            mock_cache.assert_called_once_with(
+                task, image_info, ipxe_enabled=False)
+            mock_ks_tmpl.assert_called_once_with(image_info['ks_template'][1])
+            mock_ks_file.assert_called_once_with(mock_ks_tmpl.return_value)
+            mock_prepare_ks_config.assert_called_once_with(task, image_info,
+                                                           anaconda_boot=True)
+
+    @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
+    def test_prepare(self, mock_prepare_instance):
+        node = self.node
+        node.provision_state = states.DEPLOYING
+        node.instance_info = {}
+        node.save()
+        with task_manager.acquire(self.context, node.uuid) as task:
+            task.driver.deploy.prepare(task)
+            self.assertFalse(mock_prepare_instance.called)
+
+    @mock.patch.object(pxe.PXEBoot, 'prepare_instance', autospec=True)
+    def test_prepare_active(self, mock_prepare_instance):
+        node = self.node
+        node.provision_state = states.ACTIVE
+        node.save()
+        with task_manager.acquire(self.context, node.uuid) as task:
+            task.driver.deploy.prepare(task)
+            mock_prepare_instance.assert_called_once_with(mock.ANY, task)
+
+    @mock.patch.object(pxe_utils, 'clean_up_pxe_env', autospec=True)
+    @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
+    @mock.patch.object(deploy_utils, 'try_set_boot_device', autospec=True)
+    def test_reboot_to_instance(self, mock_set_boot_dev, mock_image_info,
+                                mock_cleanup_pxe_env):
+        image_info = {'kernel': ('', '/path/to/kernel'),
+                      'ramdisk': ('', '/path/to/ramdisk'),
+                      'stage2': ('', '/path/to/stage2'),
+                      'ks_template': ('', '/path/to/ks_template'),
+                      'ks_cfg': ('', '/path/to/ks_cfg')}
+        mock_image_info.return_value = image_info
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            task.driver.deploy.reboot_to_instance(task)
+            mock_set_boot_dev.assert_called_once_with(task, boot_devices.DISK)
+            mock_cleanup_pxe_env.assert_called_once_with(task, image_info,
+                                                         ipxe_enabled=False)
+
+    @mock.patch.object(objects.node.Node, 'touch_provisioning', autospec=True)
+    def test_heartbeat_deploy_start(self, mock_touch):
+        self.node.provision_state = states.DEPLOYWAIT
+        self.node.save()
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            self.deploy.heartbeat(task, 'url', '3.2.0', None, 'start', 'msg')
+            self.assertFalse(task.shared)
+            self.assertEqual(
+                'url', task.node.driver_internal_info['agent_url'])
+            self.assertEqual(
+                '3.2.0',
+                task.node.driver_internal_info['agent_version'])
+            self.assertEqual(
+                'start',
+                task.node.driver_internal_info['agent_status'])
+            mock_touch.assert_called()
+
+    @mock.patch.object(deploy_utils, 'set_failed_state', autospec=True)
+    def test_heartbeat_deploy_error(self, mock_set_failed_state):
+        self.node.provision_state = states.DEPLOYWAIT
+        self.node.save()
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            self.deploy.heartbeat(task, 'url', '3.2.0', None, 'error',
+                                  'errmsg')
+            self.assertFalse(task.shared)
+            self.assertEqual(
+                'url', task.node.driver_internal_info['agent_url'])
+            self.assertEqual(
+                '3.2.0',
+                task.node.driver_internal_info['agent_version'])
+            self.assertEqual(
+                'error',
+                task.node.driver_internal_info['agent_status'])
+            mock_set_failed_state.assert_called_once_with(task, 'errmsg',
+                                                          collect_logs=False)
+
+    @mock.patch.object(pxe.PXEAnacondaDeploy, 'reboot_to_instance',
+                       autospec=True)
+    def test_heartbeat_deploy_end(self, mock_reboot_to_instance):
+        self.node.provision_state = states.DEPLOYWAIT
+        self.node.save()
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            self.deploy.heartbeat(task, None, None, None, 'end', 'sucess')
+            self.assertFalse(task.shared)
+            self.assertIsNone(
+                task.node.driver_internal_info['agent_url'])
+            self.assertIsNone(
+                task.node.driver_internal_info['agent_version'])
+            self.assertEqual(
+                'end',
+                task.node.driver_internal_info['agent_status'])
+            self.assertTrue(mock_reboot_to_instance.called)
+
+
 class PXEValidateRescueTestCase(db_base.DbTestCase):
 
     def setUp(self):
