@@ -61,6 +61,43 @@ class TestNodeObject(db_base.DbTestCase, obj_utils.SchemasTestMixIn):
         # Ensure the node can be serialised.
         jsonutils.dumps(d)
 
+    def test_as_dict_secure_configdrive_as_dict(self):
+        self.node.driver_info['ipmi_password'] = 'fake'
+        self.node.instance_info['configdrive'] = {'user_data': 'data'}
+        self.node.driver_internal_info['agent_secret_token'] = 'abc'
+        d = self.node.as_dict(secure=True)
+        self.assertEqual('******', d['driver_info']['ipmi_password'])
+        self.assertEqual('******', d['instance_info']['configdrive'])
+        self.assertEqual('******',
+                         d['driver_internal_info']['agent_secret_token'])
+        # Ensure the node can be serialised.
+        jsonutils.dumps(d)
+
+    def test_as_dict_secure_with_configdrive(self):
+        self.node.driver_info['ipmi_password'] = 'fake'
+        self.node.instance_info['configdrive'] = 'data'
+        self.node.driver_internal_info['agent_secret_token'] = 'abc'
+        d = self.node.as_dict(secure=True, mask_configdrive=False)
+        self.assertEqual('******', d['driver_info']['ipmi_password'])
+        self.assertEqual('data', d['instance_info']['configdrive'])
+        self.assertEqual('******',
+                         d['driver_internal_info']['agent_secret_token'])
+        # Ensure the node can be serialised.
+        jsonutils.dumps(d)
+
+    def test_as_dict_secure_with_configdrive_as_dict(self):
+        self.node.driver_info['ipmi_password'] = 'fake'
+        self.node.instance_info['configdrive'] = {'user_data': 'data'}
+        self.node.driver_internal_info['agent_secret_token'] = 'abc'
+        d = self.node.as_dict(secure=True, mask_configdrive=False)
+        self.assertEqual('******', d['driver_info']['ipmi_password'])
+        self.assertEqual({'user_data': 'data'},
+                         d['instance_info']['configdrive'])
+        self.assertEqual('******',
+                         d['driver_internal_info']['agent_secret_token'])
+        # Ensure the node can be serialised.
+        jsonutils.dumps(d)
+
     def test_as_dict_with_traits(self):
         self.fake_node['traits'] = ['CUSTOM_1']
         self.node = obj_utils.get_test_node(self.ctxt, **self.fake_node)
@@ -171,7 +208,6 @@ class TestNodeObject(db_base.DbTestCase, obj_utils.SchemasTestMixIn):
                 mock_update_node.assert_called_once_with(
                     uuid, {'properties': {"fake": "property"},
                            'driver': 'fake-driver',
-                           'driver_internal_info': {},
                            'version': objects.Node.VERSION})
                 self.assertEqual(self.context, n._context)
                 res_updated_at = (n.updated_at).replace(tzinfo=None)
@@ -221,7 +257,6 @@ class TestNodeObject(db_base.DbTestCase, obj_utils.SchemasTestMixIn):
                     {
                         'properties': {'fake': 'property'},
                         'driver': 'fake-driver',
-                        'driver_internal_info': {},
                         'version': objects.Node.VERSION,
                         'maintenance_reason':
                             maintenance_reason[
@@ -1284,6 +1319,8 @@ class TestNodePayloads(db_base.DbTestCase):
                                                        'DEPLOYING', 'DEPLOY')
         self._test_node_payload(payload)
         self.assertEqual(self.node.instance_info, payload.instance_info)
+        self.assertEqual(self.node.driver_internal_info,
+                         payload.driver_internal_info)
         self.assertEqual('DEPLOY', payload.event)
         self.assertEqual('AVAILABLE', payload.previous_provision_state)
         self.assertEqual('DEPLOYING', payload.previous_target_provision_state)
